var menuService = require('./menu');

var basketService = {};

/**
 * Export the config for the env
 */
basketService.config = menuService.config;

/**
 * Use this method to create an instance of an basket.
 * @returns {{item: Array, totalPrice: number}}
 */
basketService.instance = function () {
    var basket = {
        items: [],
        totalPrice: 0,
        orderType: '',
        firstName: '',
        lastName: '',
        telNumber: '',
        mail: '',
        numberOfPeople: '',
        day: '',
        month: '',
        year: '',
        hour: '',
        min: '',
        notes: ''
    };

    /**
     * Returns the basket instance as json for placing orders.
     */
    basket.toJSON = function () {
        return {
            'items': basket.items,
            'total_price': basket.totalPrice,
            'order_type': basket.orderType,
            'first_name': basket.firstName,
            'last_name': basket.lastName,
            'tel_number': basket.telNumber,
            'mail': basket.mail,
            'number_of_people': basket.numberOfPeople,
            'day': basket.day,
            'month': basket.month,
            'year': basket.year,
            'hour': basket.hour,
            'min': basket.min,
            'notes': basket.notes
        };
    };
    
    basket.setFirstName = function (_name) {
        basket.firstName = _name;
    };

    basket.setNotes = function (_notes) {
        basket.notes = _notes;
    };

    basket.setLastName = function (_name) {
        basket.lastName = _name;
    };

    basket.setTelNumber = function (_tel) {
        basket.telNumber = _tel;
    };

    basket.setMail = function (_mail) {
        basket.mail = _mail;
    };

    basket.setNumberOfPeople = function (_number) {
        basket.numberOfPeople = _number;
    };

    basket.setDay = function (_day) {
        basket.day = _day;
    };

    basket.setMonth = function (_m) {
        basket.month = _m;
    };

    basket.setYear = function (_y) {
        basket.year = _y;
    };

    basket.setHours = function (_h) {
        basket.hour = _h;
    };

    basket.setMinutes = function (_m) {
        basket.min = _m;
    };

    /**
     * @returns {string} returns the current order type, like reservation or takeAway
     */
    basket.getOrderType = function () {
      return basket.orderType;
    };

    /**
     * Use this method to set the order type for this basket.
     *
     * The following order types are supported:
     *  - reservation, use basketService.reservationType() to get the correct type
     *  - takeAway, use basketService.takeAwayType() to get the correct type
     *
     * @param _type must not be null, must be a correct order type
     */
    basket.setOrderType = function (_type) {
        if(_type != null && (_type == 'reservation' || _type == 'takeAway' || _type == 'discountOnly')) {
            basket.orderType = _type
        }
    };

    /**
     * TODO: add support for selection_items
     *
     * Use this method to add one menu item to the basket.
     * @param _menu_item_id must not be null, must be a valid menu item id (UUID)
     * @param _selection_items_ must not be null, must be an array of menu item ids for selection
     * @param onSuccess(item) is called when adding the menu item to the basket was successful
     * @param onFail(message) is called on fail or if adding the items fails
     * @param onError(err) is called on error or if adding the item fails
     */
    basket.addMenuItem = function (_menu_item_id, _selection_items_, onSuccess, onFail, onError) {
        /**
         * Check if the given items exits
         */
        menuService.getMenuItem(_menu_item_id,
            function (_item) {
                var found = false;

                for(var i = 0; i < basket.items.length; i++) {
                    var currItem = basket.items[i];

                    if(currItem != null && currItem.item.id == _item.id) {
                        // we found the item
                        found = true;

                        // we increase the quantity by one
                        basket.items[i].quantity = basket.items[i].quantity + 1;

                        // we stop the for loop
                        break;
                    }
                }

                /**
                 * We add the menu item for the first time to the basket
                 */
                if(!found) {
                    basket.items.push({ item: _item, quantity: 1 });
                }

                // we increase the totalPrice
                // convert to number, because the price is saved an an string
                var price = Number(_item.price);
                basket.totalPrice = basket.totalPrice + price;

                found = false;
                return onSuccess(_item);
            }, onFail, onError);
    };

    /**
     * TODO: add support for selection_items
     *
     * Use this method to remove items from the menu.
     * @param _menu_item_id must not be null, must be a valid menu item id (UUID)
     * @param _selection_items_ must not be null, must be valid array of selection items ids
     * @param onSuccess(removedMenuItem) is called on successfully removing the item
     * @param onFail(message) is called on fail or if removing item fails
     * @param onError(err) is called on error or if removing item fails
     */
    basket.removeMenuItem = function (_menu_item_id, _selection_items_, onSuccess, onFail, onError) {
        if(basket.quantity() == 0) {
            /**
             * The basket is empty
             */
            return onFail('The basket is empty.');
        } else {
            for(var i = 0; i < basket.items.length; i++) {
                var currItem = basket.items[i];

                /**
                 * We found the item which should be removed
                 */
                if(currItem != null && currItem.item.id == _menu_item_id) {
                    /**
                     * Decrease the quantity of the items
                     */
                    if(currItem.quantity > 1) {
                        basket.items[i].quantity = basket.items[i].quantity - 1;
                    }
                    /**
                     * Completely remove the item from the basket
                     */
                    else {
                        currItem = (basket.items.splice(i, 1))[0];
                    }

                    /**
                     * Recalculate the current totalPrice
                     */
                    var price = Number(currItem.item.price);
                    basket.totalPrice = basket.totalPrice - price;

                    return onSuccess(currItem.item);
                }
            }

            return onFail('The given menu item was not found.');
        }
    };

    /**
     * @returns {number} returns the total quantity size of the basket, meaning all items are count (even if one items occurres multiple times).
     */
    basket.quantity = function () {
        var q = 0;

        for(var i = 0; i < basket.items.length; i++) {
            q = q + (basket.items[i].quantity);
        }

        return q;
    };

    /**
     * @returns {number} returns the number of unique items in the basket, meaning each items is count as an distinct entry (no mult. entries are count)
     */
    basket.size = function () {
        return basket.items.length;
    };

    /**
     * @returns {boolean} returns true if the basket is empty else false
     */
    basket.isEmpty = function () {
        return basket.quantity() == 0;
    };

    return basket;
};

/**
 * @returns {string} returns the reservation type string
 */
basketService.reservationType = function () {
  return 'reservation';
};

/**
 * @returns {string} returns the take away type string
 */
basketService.takeAwayType = function () {
    return 'takeAway';
};

/**
 * @returns {string} returns the discount only type string, only if a user wants to discount their order
 */
basketService.discountType = function () {
    return 'discountOnly';
};

/**
 * Use this method to format a price of lowest valute to a formatted printable string
 * Example: assert.equal('100,00 €', basketService.formatPrice(10000, 2, "€", ".", ","));
 *
 * @param number must not be null, must be a numeric string, must be in lowest valute value like 1090, not like 10.90
 * @param places fraction
 * @param symbol the currency, must not be null
 * @param thousand the thousnd separator, must not be null
 * @param decimal the decimal separator, must not be null
 * @returns {string} returns a printable formatted price as string
 */
basketService.formatPrice = function(number, places, symbol, thousand, decimal) {
    if(number && !isNaN(parseFloat(number)) && number > 0) {
        number = number / 100;
    } else {
        return '';
    }

    places = !isNaN(places = Math.abs(places)) ? places : 2;
    symbol = symbol !== undefined ? symbol : "$";
    thousand = thousand || ".";
    decimal = decimal || ",";

    var negative = number < 0 ? "-" : "",
        i = parseInt(number = Math.abs(+number || 0).toFixed(places), 10) + "",
        j = (j = i.length) > 3 ? j % 3 : 0;
    return negative + (j ? i.substr(0, j) + thousand : "") + i.substr(j).replace(/(\d{3})(?=\d)/g, "$1" + thousand) + (places ? decimal + Math.abs(number - i).toFixed(places).slice(2) : "") + ' ' + symbol;
};

/**
 * Use this method to get the order code out from the order id
 * @param _order_id must not be null, must be a valid order id (UUID)
 * @returns {*} returns a valid order code from the order id
 */
basketService.getOrderCode = function (_order_id) {
    if(_order_id != null && _order_id.length > 0) {
        var code = _order_id.split('-');

        if (code != null && code.length > 0) {
            return code[0];
        }
    }

    return '';
};

module.exports = basketService;

