var config = require('./config');
var request = require('./request');

var clientService = {};

/**
 * Export the config for the env
 */
clientService.config = config;

/**
 * Use this method to get a client (account) by token.
 * @param _token must not be null, must be a valid client token
 * @param onSuccess(client, account) is called on successfully retrieving the client's account
 * @param onFail(message) is called when retrieving fails
 * @param onError(error) is called on error or if finding the client's accounts fails
 */
clientService.getClient = function (_token, onSuccess, onFail, onError) {
    request.getJSON(_token, config.api('/clients'),
        function (body) {
            if (onSuccess != null) {
                return onSuccess(body.client, body.account);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to get a client (account) by token.
 * @param _token must not be null, must be a valid client token
 * @param onSuccess(client, account) is called on successfully retrieving the client's account
 * @param onFail(message) is called when retrieving fails
 * @param onError(error) is called on error or if finding the client's accounts fails
 */
clientService.setClient = function (_token, _first_name, _last_name, _title, _tel_number, onSuccess, onFail, onError) {
    if (_token == null) {
        return onError('Invalid token was given.');
    }

    request.postJSON(_token,
        config.api('/clients'),
        {
            first_name: _first_name,
            last_name: _last_name,
            title: _title,
            tel_number: _tel_number
        },
        function (body) {
            if (onSuccess != null) {
                return onSuccess();
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to get all locations owned by the client.
 * @param _token must not be null, must be a valid client token
 * @param onSuccess(locations) is called when finding all locations by the client was successful
 * @param onFail(message) is called when finding the locations fails
 * @param onError(err) is called on error or if finding the client fails
 */
clientService.getClientsLocations = function (_token, onSuccess, onFail, onError) {
    request.getJSON(_token, config.api('/clients/locations'),
        function (body) {
            if (onSuccess != null) {
                return onSuccess(body.locations);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to get one the location by id. The client must be the owner of the location in order
 * to retrieve it.
 * @param _token must not be null, must be a valid client token
 * @param _location_id must not be null, must be a valid location id (UUID)
 * @param onSuccess(location) is called on successfully retrieving client
 * @param onFail(message) is called when retrieving the location fails
 * @param onError(err) is called on error or when retrieving the location fails
 */
clientService.getClientsLocation = function (_token, _location_id, onSuccess, onFail, onError) {
    request.getJSON(_token, config.api('/clients/locations/' + _location_id),
        function (body) {
            if (onSuccess != null) {
                return onSuccess(body.location);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to get all orders by state from a given location by id (UUID).
 * The client must be the owner of the location.
 * @param _token must not be null, must be a valid client token
 * @param _location_id must not be null, must be a valid location_id
 * @param _status may be null, may be pending, denied or confirmed, the status will define the type of order to return
 * @param _pos may be null, starting position of the client's orders
 * @param _length may be null, the length to return, maximum 30
 * @param onSuccess(orders) is called when successfully retrieving the client's orders
 * @param onFail (message) is called on failure
 * @param onError (error) is called on error
 */
clientService.getOrders = function (_token, _location_id, _status, _pos, _length, onSuccess, onFail, onError) {
    if (_token == null) {
        return onError(new Error('Invalid token was given.'));
    }

    var query = '';

    if(_pos != null && _length != null && _status != null) {
        query = '?pos=' + _pos + '&length=' + _length + '&status=' + _status;
    }

    request.getJSON(_token, config.api('/clients/locations/' + _location_id + '/orders/' + query),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.orders);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to get one order by id (UUID).
 * @param _token must not be null, must be a valid client token
 * @param onSuccess(order, coupon, items) is called when successfully retrieving the user's orders
 * @param onFail (message) is called on failure
 * @param onError (error) is called on error
 */
clientService.getOrder = function (_token, _order_id, onSuccess, onFail, onError) {
    if (_token == null) {
        return onError('Invalid token was given.');
    }

    request.getJSON(_token, config.api('/clients/orders/' + _order_id),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.order, body.coupon, body.items);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to get the current order status for a given location and client.
 * The client must be the owner of the location.
 * @param _token must not be null, must be a valid client token
 * @param _location_id must not be null, must be a valid location_id
 * @param onSuccess(status) is called when successfully retrieving the client's orders
 * @param onFail (message) is called on failure
 * @param onError (error) is called on error
 */
clientService.getOrdersStatus = function (_token, _location_id, onSuccess, onFail, onError) {
    if (_token == null) {
        return onError('Invalid token was given.');
    }

    request.getJSON(_token, config.api('/clients/locations/' + _location_id + '/orders/status'),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.status);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to check if the given order is in the given state
 * @param _order_id must not be null, must be a valid order id (UUD)
 * @param _status must not be null or empty, must be a valid order status
 * @param onSuccess() is called when the given order is in the given state
 * @param onFail() is called when the given order is NOT in the given state
 * @param onError(err) is called on error
 */
clientService.isOrderInState = function (_order_id, _status, onSuccess, onFail, onError) {
    if(_status == null) {
        return onError(new Error('Given status is invalid'));
    }

    request.getJSON(null, config.api('/public/clients/orders/' + _order_id + '/state?status=' + _status),
        function (body) {
            if(onSuccess != null) {
                return onSuccess();
            }
        },
        function (message) {
            return onFail();
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to confirm an order (as an client).
 * @param _token must not be null, must be a valid client token
 * @param _order_id must not be null, must be a valid pending order_id
 * @param onSuccess(order) is called on successfully confirming the order
 * @param onFail(message) is called when confirming the order fails
 * @param onError(err) is called on error or when confirming the order fails
 */
clientService.confirmOrder = function (_token, _order_id, onSuccess, onFail, onError) {
    if (_token == null) {
        return onError('Invalid token was given.');
    }

    request.postJSON(_token,
        config.api('/clients/orders/' + _order_id + '/confirm'),
        { /* empty body */ },
        function (body) {
            if (onSuccess != null) {
                return onSuccess(body.order);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to deny an order (as an client).
 * @param _token must not be null, must be a valid client token
 * @param _order_id must not be null, must be a valid pending order_id
 * @param onSuccess(order) is called on successfully denying the order
 * @param onFail(message) is called when denying the order fails
 * @param onError(err) is called on error or when denying the order fails
 */
clientService.denyOrder = function (_token, _order_id, onSuccess, onFail, onError) {
    if (_token == null) {
        return onError('Invalid token was given.');
    }

    request.postJSON(_token,
        config.api('/clients/orders/' + _order_id + '/deny'),
        { /* empty body */ },
        function (body) {
            if (onSuccess != null) {
                return onSuccess(body.order);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to deny an order without a authentication.
 * The secured denyOrder will return the order object back, the public won't
 *
 * @param _order_id must not be null, must be a valid pending order_id
 * @param onSuccess() is called on successfully denying the order
 * @param onFail(message) is called when denying the order fails
 * @param onError(err) is called on error or when denying the order fails
 */
clientService.publicOrderDeny = function (_order_id, onSuccess, onFail, onError) {
    request.postJSON(null,
        config.api('/public/clients/orders/' + _order_id + '/deny'),
        { /* empty body */ },
        function (body) {
            if (onSuccess != null) {
                return onSuccess();
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to confirm an order without a authentication.
 * The secured confirmOrder will return the order object back, the public won't
 *
 * @param _order_id must not be null, must be a valid pending order_id
 * @param onSuccess() is called on successfully denying the order
 * @param onFail(message) is called when denying the order fails
 * @param onError(err) is called on error or when denying the order fails
 */
clientService.publicOrderConfirm = function (_order_id, onSuccess, onFail, onError) {
    request.postJSON(null,
        config.api('/public/clients/orders/' + _order_id + '/confirm'),
        { /* empty body */ },
        function (body) {
            if (onSuccess != null) {
                return onSuccess();
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

module.exports = clientService;

