var config = require('./config');
var request = require('./request');

var discountService = {};

/**
 * Export the config for the env
 */
discountService.config = config;

/**
 * Use this method to get an discount by id.
 * @param _id must not be null, must be a valid discount UUID
 * @param onSuccess(discount) is called on successfully retrieving discount
 * @param onFail(message) is called on failure
 * @param onError(error) is called on error
 */
discountService.getDiscount = function (_id, onSuccess, onFail, onError) {
    request.getJSON(null, config.api('/public/discounts/' + _id),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.discount);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to add an discount to an location.
 * @param _token must not be null, must be a valid client token
 * @param _location_id must not be null, must be a valid location id (UUID)
 * @param _discount must not be null, _discount >= 1 or <= 100 (100 == free meal)
 * @param _items must not be null, may be empty array
 * @param excluded_items may be null
 * @param _start_year
 * @param _start_month (1 == Jan .. 12 == Dez)
 * @param _start_day_of_month (1 == first day of month 30/31 == last day month)
 * @param _start_hour 24 h format
 * @param _start_min 0 .. 59 min
 * @param _end_year
 * @param _end_month (1 == Jan .. 12 == Dez)
 * @param _end_day_of_month (1 == first day of month 30/31 == last day month)
 * @param _end_hour 25 h format
 * @param _end_min 0 .. 59 min
 * @param onSuccess(discount, location) is called when successfully adding discount
 * @param onFail(message) is called when adding discount fails
 * @param onError(err) is called on error of if adding discount fails
 * @returns {*}
 */
discountService.addDiscount = function (_token,
                                        _location_id,
                                        _discount,
                                        _items,
                                        _excluded_items,
                                        _start_year,
                                        _start_month,
                                        _start_day_of_month,
                                        _start_hour,
                                        _start_min,
                                        _end_year,
                                        _end_month,
                                        _end_day_of_month,
                                        _end_hour,
                                        _end_min, onSuccess, onFail, onError) {

    if (_token == null) {
        return onError(new Error('Invalid token was given.'));
    }

    request.putJSON(_token,
        config.api('/discounts'),
        {
            'location_id': _location_id,
            'discount': _discount,
            'items': _items,
            'excluded_items': _excluded_items,
            'start_year': _start_year,
            'start_month': _start_month,
            'start_day_of_month': _start_day_of_month,
            'start_hour': _start_hour,
            'start_min': _start_min,
            'end_year': _end_year,
            'end_month': _end_month,
            'end_day_of_month': _end_day_of_month,
            'end_hour': _end_hour,
            'end_min': _end_min
        },
        function (body) {
            if (onSuccess != null) {
                return onSuccess(body.discount, body.location);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to get all discounts from a specified location (by id)
 * @param _location_id must not be null, must be a valid location's UUID
 * @param _pos may be null, use this to set the retrieving offset of the data-set
 * @param _length may by null, use this to set the maximal data-set length
 * @param _active if this field is 'true', only discounts that are active (now() <= discount.time) are selected, meaning only discounts that are valid (active) from today are selected
 *                          If this field is 'false', then all discounts are selected (also the ones that already passed)
 * @param onSuccess(discounts) is called on successfully retrieving the location's discounts
 * @param onFail(message) is called on failure
 * @param onError(error) is called on error
 */
discountService.getLocationDiscounts = function (_location_id, _pos, _length, _active, onSuccess, onFail, onError) {
    var query = '';

    if(_pos != null && _length != null && _active != null) {
        query = '?pos=' + _pos + '&length=' + _length + '&active=' + _active;
    }

    request.getJSON(null, config.api('/public/discounts/all/' + _location_id + query),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.discounts);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to delete (archive) an discount by id (UUID).
 * @param _token must not be null, must be a valid client token
 * @param _discount_id must not be null, must be a valid discount id (UUID)
 * @param onSuccess() successfully deleted discount
 * @param onFail(message) is called when deleting discount fails
 * @param onError is called on error of if deleting discount fails
 * @returns {*}
 */
discountService.deleteDiscount = function (_token, _discount_id, onSuccess, onFail, onError) {
    if (_token == null) {
        return onError('Invalid token was given');
    }

    request.deleteJSON(_token, config.api('/discounts/' + _discount_id),
        function (body) {
            if(onSuccess != null) {
                return onSuccess();
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to retrieve discounts from a given location with a range inclusion selection.
 * All discounts that are have the start and the end dates in range will be selected. This method
 * range selection uses strict inclusion, meaning the start AND end dates must be within or equals
 * the discount in order to be selected.
 *
 * The locations's timezone is used for the inclusion selection.
 *
 * @param _location_id must not be null
 * @param _limit must not be null, must be > 0
 * @param _offset must not be null
 * @param _start_year must not be null, must be a valid year for 24 h format
 * @param _start_month must not be null, must be a valid 24 h format, starting from 1 == Jan, 12 == Dez
 * @param _start_day_of_month must not be null, must be a valid 24 h format, starting from 1 == first day of month, 31 == last day of month
 * @param _start_hour must not be null, must be a valid 24 h format, must be between 0 and 23
 * @param _start_min must not be null, must be a valid 24 h format, must be between 0 and 59
 * @param _end_year must not be null, must be a valid year for 24 h format
 * @param _end_month must not be null, must be a valid 24 h format, starting from 1 == Jan, 12 == Dez
 * @param _end_day_of_month must not be null, must be a valid 24 h format, starting from 1 == first day of month, 31 == last day of month
 * @param _end_hour must not be null, must be a valid 24 h format, must be between 0 and 23
 * @param _end_min must not be null, must be a valid 24 h format, must be between 0 and 59
 *
 * @param onSuccess(discounts) is called on successfully retrieving discounts, may be empty array
 * @param onFail(message) is called when retrieving discounts fails
 * @param onError(err) is called on error or if retrieving discounts fails
 */
discountService.getLocationDiscountsByRangeInclusion = function (_location_id, _limit, _offset, _start_year, _start_month, _start_day_of_month, _start_hour, _start_min, _end_year, _end_month, _end_day_of_month, _end_hour, _end_min, onSuccess, onFail, onError) {
    if(_location_id == null) {
        return onError(new TypeError('Invalid location id found (null).'));
    }

    var query = '';

    var limit = _limit != null ? _limit : '30';
    var offset = _offset != null ? _offset : '0';

    if(_start_year != null && _start_month != null && _start_day_of_month != null && _start_hour != null && _start_min != null && _end_year != null && _end_month != null && _end_day_of_month != null && _end_hour != null && _end_min != null) {
        query = '?start_year=' + _start_year
                +   '&end_year=' + _end_year
                +   '&start_month=' + _start_month
                +   '&end_month=' + _end_month
                +   '&start_day_of_month=' + _start_day_of_month
                +   '&end_day_of_month=' + _end_day_of_month
                +   '&start_hour=' + _start_hour
                +   '&end_hour=' + _end_hour
                +   '&start_min=' + _start_min
                +   '&end_min=' + _end_min
                +   '&pos=' + offset
                +   '&length=' + limit;
    } else{
        return onError(new TypeError('Given date arguments are invalid'));
    }

    request.getJSON(null, config.api('/public/discounts/locations/' + _location_id + '/range' + query),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.discounts);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to get a locations discounts. Where the given date is in the range (start <= date <= end) of the discounts
 * @param _location_id must not be null
 * @param _limit must not be null, must be > 0
 * @param _offset must not be null
 * @param _start_year must not be null, must be a valid year for 24 h format
 * @param _start_month must not be null, must be a valid 24 h format, starting from 1 == Jan, 12 == Dez
 * @param _start_day_of_month must not be null, must be a valid 24 h format, starting from 1 == first day of month, 31 == last day of month
 * @param _start_hour must not be null, must be a valid 24 h format, must be between 0 and 23
 * @param _start_min must not be null, must be a valid 24 h format, must be between 0 and 59
 *
 * @param onSuccess(discounts) is called on successfully retrieving discounts, may be empty array
 * @param onFail(message) is called when retrieving discounts fails
 * @param onError(err) is called on error or if retrieving discounts fails
 */
discountService.getLocationDiscountsByDate = function (_location_id, _limit, _offset, _start_year, _start_month, _start_day_of_month, _start_hour, _start_min, onSuccess, onFail, onError) {
    if(_location_id == null) {
        return onError(new TypeError('Invalid location id found (null).'));
    }

    var query = '';

    var limit = _limit != null ? _limit : '30';
    var offset = _offset != null ? _offset : '0';

    if(_start_year != null && _start_month != null && _start_day_of_month != null && _start_hour != null && _start_min != null) {
        query = '?start_year=' + _start_year
            +   '&end_year=' + _start_year
            +   '&start_month=' + _start_month
            +   '&end_month=' + _start_month
            +   '&start_day_of_month=' + _start_day_of_month
            +   '&end_day_of_month=' + _start_day_of_month
            +   '&start_hour=' + _start_hour
            +   '&end_hour=' + _start_hour
            +   '&start_min=' + _start_min
            +   '&end_min=' + _start_min
            +   '&pos=' + offset
            +   '&length=' + limit;
    } else{
        return onError(new TypeError('Given date arguments are invalid'));
    }

    request.getJSON(null, config.api('/public/discounts/locations/' + _location_id + '/range' + query),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.discounts);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

module.exports = discountService;

