var config = require('./config');
var request = require('./request');

var locationService = {};

/**
 * Export the config for the env
 */
locationService.config = config;

/**
 * Use this method to get a location by id.
 * @param _id Location's id, must not be null, must be valid locations UUID
 * @param onSuccess (location)
 *
 * A locations object has the following structure:
 *
 * { id: '1e042f7e-de16-4e6e-b077-d18921cb6aa1',
    name: 'URBANs Lounge',
    full_name: 'URBANs Lounge Restaurant',
    impressum: 'URBANs Lounge Restaurant, Antonie-Alt-Gasse 2/1/10A, A-1100 Wien',
    front_image: null,
    state: 'AT',
    continent: 'EU',
    city: 'Vienna',
    street: 'Antonie-Alt-Gasse 2/1/10A',
    zip: '1010',
    tel_number: '+43 1 208 7012',
    mail: 'office@urbans.wien',
    open: { opening: [Object] },
    client_id: 'fcc98b65-f1e9-4f8f-8d2c-53066acec773',
    lat: '48.1819123',
    lng: '16.381373',
    rad: '10',
    tags:
     [ 'Italian',
       'Continental',
       'Austrian',
       'Mediterranean',
       'Vegan',
       'Vegetarian',
       'Steakhouses',
       'Steak',
       'Mittagessen' ],
    types:
     [ 'Italian',
       'Continental',
       'Austrian',
       'Mediterranean',
       'American' ],
    location_type_id: 'restaurant',
    createdAt: null,
    updatedAt: null,
    content: { content: [Object] },
    rating: '5',
    description: 'Das Urban´s der neue Hotspot - ein Restaurant bei dem Genuss und gesunde Ernährung kein Widerspruch sind. Entdecken Sie unsere aromatische und delikate Küche in kosmopolitischem Flair. Gustieren Sie in einem ungezwungenen Ambiente und lernen Sie unsere vielfältige internationale Küche kennen - vom gesunden Start in den Morgen, dem kleinen Lunch für zwischendurch oder den köstlichen Dinner-Variationen. Verweilen Sie im Anschluss in unserer stylischen Urban’s Lounge beim chilligen Afterwork-Drink. Raucher sind uns in unserem überdachten Gastgarten willkommen. Wir hoffen, auch Sie schon bald im Urban’s begrüßen zu dürfen. Stay tuned…',
    cover_image: null,
    local_timezone: 'Europe/Vienna',
    min_arrival_timespan_in_minutes: '30',
    max_reservation_capacity: 16,
    smoking_zone: false,
    language: 'de',
    allow_discount: true,
    allow_reservation: true,
    allow_pre_order: true }
 *
 * @param onFail (message)
 * @param onError (error)
 */
locationService.getLocation = function (_id, onSuccess, onFail, onError) {
    request.getJSON(null, config.api('/public/locations/' + _id),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.location);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to get the location's opening hours from now() till (now() + days).
 * This method is useful when you want to show the working days and hours of the location.
 *
 * @param _id must not be null, must be a valid location id (UUID)
 * @param _days must not be null, must be a valid number > 0
 * @param _timespan
 * @param _step
 * @param _lang
 * @param onSuccess(openings) is called when retrieving the openings was sucessful
 *
 * The openings object has a structure like this:
 *  "openings": [
 {
   "is_today": true,
   "is_open": true,
   "opening_hours": [
     {
       "hour": 16,
       "min": 0,
       "passed": true
     },
     {
       "hour": 16,
       "min": 15,
       "passed": true
     },
     {
       "hour": 16,
       "min": 30,
       "passed": true
     },
     {
       "hour": 16,
       "min": 45,
       "passed": true
     },
     {
       "hour": 22,
       "min": 0
     },
     {
       "hour": 22,
       "min": 15
     }
   ],
   "day": 7,
   "month": 2,
   "year": 2017,
   "time_zone": "Europe/Vienna",
   "language": "de",
   "formatted_date": "Di., 7. Febr. 2017"
 },
 *
 *
 * @param onFail(message) is called on fails
 * @param onError(err) is called on error
 */
locationService.getOpenings = function (_id, _days, _timespan, _step, _lang, onSuccess, onFail, onError) {
    if(_id == null) {
        return onFail(new Error('You must provide a location id.'));
    }

    var query = '';

    if(_days != null && _timespan != null && _step != null && _lang != null) {
        query = '?days=' + _days + '&step=' + _step + '&timespan=' + _timespan + '&lang=' + _lang;
    }

    request.getJSON(null, config.api('/public/locations/' + _id + '/openings' + query),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.openings);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to get the cover image of the location. You might specify which
 * cover image to request to, from the local baluu system or from facebook by
 * setting the type.
 *
 * @param _id must not be null, must be a valid location id (UUID)
 * @param _type must not be null, may be: baluu, facebook
 * @param onSuccess(url) is called on successfully retrieving the cover url
 * @param onFail(message) is called on failure
 * @param onError(err) is called on error or if retrieving the cover url fails
 */
locationService.getLocationCover = function (_id, _type, onSuccess, onFail, onError) {
    var query = '';

    if(_type != null) {
        query = '?type=' + _type;
    }

    request.getJSON(null, config.api('/public/locations/' + _id + '/cover' + query),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.url);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to add a new location by a client (token).
 * @param _token must not be null, must be a valid client token
 * @param _name
 * @param _location_type
 * @param _full_name
 * @param _front_image
 * @param _state
 * @param _continent
 * @param _city
 * @param _street
 * @param _zip
 * @param _tel_number
 * @param _types
 * @param _mail
 * @param _open
 * @param _lat
 * @param _lng
 * @param _rad
 * @param _tags
 * @param _content
 * @param _rating
 * @param _description
 * @param _cover_image
 * @param _local_time_zone
 * @param _max_reservation_capacity
 * @param _min_arrival_timespan_in_minutes
 * @param _smoking_zone
 * @param _language
 * @param _allow_discount
 * @param _allow_reservation
 * @param _allow_pre_order
 * @param onSuccess(location) is called when successfully adding location
 * @param onFail(message) is called when adding the location failed
 * @param onError(err) is called on error
 */
locationService.addLocation = function (_token,
                                        _name,
                                        _location_type,
                                        _full_name,
                                        _front_image,
                                        _state,
                                        _continent,
                                        _city,
                                        _street,
                                        _zip,
                                        _tel_number,
                                        _types,
                                        _mail,
                                        _open,
                                        _lat,
                                        _lng,
                                        _rad,
                                        _tags,
                                        _content,
                                        _rating,
                                        _description,
                                        _cover_image,
                                        _local_time_zone,
                                        _max_reservation_capacity,
                                        _min_arrival_timespan_in_minutes,
                                        _smoking_zone,
                                        _language,
                                        _allow_discount,
                                        _allow_reservation,
                                        _allow_pre_order, onSuccess, onFail, onError) {
    if (_token == null) {
        return onError(new Error('Invalid token was given.'));
    }

    request.putJSON(_token,
        config.api('/locations'),
        {
            'name': _name,
            'location_type': _location_type,
            'full_name': _full_name,
            'front_image': _front_image,
            'state': _state,
            'continent': _continent,
            'city': _city,
            'street': _street,
            'zip': _zip,
            'tel_number': _tel_number,
            'types': _types,
            'mail': _mail,
            'open': _open,
            'lat': _lat,
            'lng': _lng,
            'rad': _rad,
            'tags': _tags,
            'content': _content,
            'rating': _rating,
            'description': _description,
            'cover_image': _cover_image,
            'local_time_zone': _local_time_zone,
            'max_reservation_capacity': _max_reservation_capacity,
            'min_arrival_timespan_in_minutes': _min_arrival_timespan_in_minutes,
            'smoking_zone': _smoking_zone,
            'language': _language,
            'allow_discount': _allow_discount,
            'allow_reservation': _allow_reservation,
            'allow_pre_order': _allow_pre_order,
        },
        function (body) {
            if (onSuccess != null) {
                return onSuccess(body.location);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to update an location by id (UUID).
 *
 * Attention: This update is an OVERWRITE update, meaning all data fields are overwritten with the given fields (even if they are empty).
 * Therefore get the location data with getLocation(...) first, populate the fields and change the needed fields that should be updated.
 *
 * @param _token
 * @param _location_id
 * @param _name
 * @param _location_type
 * @param _full_name
 * @param _front_image
 * @param _state
 * @param _continent
 * @param _city
 * @param _street
 * @param _zip
 * @param _tel_number
 * @param _types
 * @param _mail
 * @param _open
 * @param _lat
 * @param _lng
 * @param _rad
 * @param _tags
 * @param _content
 * @param _rating
 * @param _description
 * @param _cover_image
 * @param _local_time_zone Local timezone of the location, default 'Europa/Vienna'
 * @param _max_reservation_capacity Max. capacity of an table reservation
 * @param _min_arrival_timespan_in_minutes The minimum arrival time span in minutes before an pre order (meal preparation) is needed. For example the location needs 20 minutes to prepare a meal,
 *                                         therefore the reservation time must >= (now() + _min_arrival_timespan_in_minutes)
 * @param _smoking_zone True if the location support smoking zones
 * @param _language Language tag, default 'de'
 * @param _allow_discount True if the location accepts discounts, else false
 * @param _allow_reservation True if the location accepts reservation orders, else false
 * @param _allow_pre_order True if the location accepts pre orders, else false
 * @param _min_reservation_cancellation_time The minimum time span in minutes when the user is able to cancel an reservation only order
 * @param _min_pre_order_cancellation_time The minimum time span in minutes when the user is able to cancel an pre_order order
 * @param _min_confirmation_time The minimum time span in minutes when orders are automatically confirmed.
 *                               For example, if a user books a table reservation 3 days ahead, then we may confirm an order automatically (client must not confirm the order) due to the long time period
 * @param _min_voice_call_time The minimum time span to execute a voice call before an order is placed.
 *                             For example, if the user places an reservation at 14.00 and the current (now) time is 13.55, then we will execute a voice call, instead of sending sms
 *
 * @param onSuccess(location) is called when updating the location was successful
 * @param onFail(message) is called on fail or if updating the location fails
 * @param onError(err) is called on error or if updating the location fails
 * @returns {*}
 */
locationService.updateLocation = function (_token,
                                           _location_id,
                                           _name,
                                           _location_type,
                                           _full_name,
                                           _front_image,
                                           _state,
                                           _continent,
                                           _city,
                                           _street,
                                           _zip,
                                           _tel_number,
                                           _types,
                                           _mail,
                                           _open,
                                           _lat,
                                           _lng,
                                           _rad,
                                           _tags,
                                           _content,
                                           _rating,
                                           _description,
                                           _cover_image,
                                           _local_time_zone,
                                           _max_reservation_capacity,
                                           _min_arrival_timespan_in_minutes,
                                           _smoking_zone,
                                           _language,
                                           _allow_discount,
                                           _allow_reservation,
                                           _allow_pre_order,
                                           _min_reservation_cancellation_time,
                                           _min_pre_order_cancellation_time,
                                           _min_confirmation_time,
                                           _min_voice_call_time, onSuccess, onFail, onError) {
    if (_token == null) {
        return onError(new Error('Invalid token was given.'));
    }

    request.postJSON(_token,
        config.api('/locations/' + _location_id),
        {
            'name': _name,
            'location_type': _location_type,
            'full_name': _full_name,
            'front_image': _front_image,
            'state': _state,
            'continent': _continent,
            'city': _city,
            'street': _street,
            'zip': _zip,
            'tel_number': _tel_number,
            'types': _types,
            'mail': _mail,
            'open': _open,
            'lat': _lat,
            'lng': _lng,
            'rad': _rad,
            'tags': _tags,
            'content': _content,
            'rating': _rating,
            'description': _description,
            'cover_image': _cover_image,
            'local_time_zone': _local_time_zone,
            'max_reservation_capacity': _max_reservation_capacity,
            'min_arrival_timespan_in_minutes': _min_arrival_timespan_in_minutes,
            'smoking_zone': _smoking_zone,
            'language': _language,
            'allow_discount': _allow_discount,
            'allow_reservation': _allow_reservation,
            'allow_pre_order': _allow_pre_order,
            'min_reservation_cancellation_time': _min_reservation_cancellation_time,
            'min_pre_order_cancellation_time': _min_pre_order_cancellation_time,
            'min_confirmation_time': _min_confirmation_time,
            'min_voice_call_time': _min_voice_call_time
        },
        function (body) {
            if (onSuccess != null) {
                return onSuccess(body.location);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to get a location by id including its discounts
 * @param _id Location's id, must not be null, must be valid locations UUID
 * @param _pos may be null, the starting position of the discount's data-set, default 0
 * @param _length may be null, the max. limit of the discount's data-set, default and maximum 30
 * @param _active my be null, if active is true then only discounts that are valid at today (>= now()) are returned. If active if false
 * also discounts that already passed (older discounts) are also included, default true
 *
 * @param onSuccess (location, discounts)
 *
 * A locations object has the following structure:
 *
 * { id: '1e042f7e-de16-4e6e-b077-d18921cb6aa1',
    name: 'URBANs Lounge',
    full_name: 'URBANs Lounge Restaurant',
    impressum: 'URBANs Lounge Restaurant, Antonie-Alt-Gasse 2/1/10A, A-1100 Wien',
    front_image: null,
    state: 'AT',
    continent: 'EU',
    city: 'Vienna',
    street: 'Antonie-Alt-Gasse 2/1/10A',
    zip: '1010',
    tel_number: '+43 1 208 7012',
    mail: 'office@urbans.wien',
    open: { opening: [Object] },
    client_id: 'fcc98b65-f1e9-4f8f-8d2c-53066acec773',
    lat: '48.1819123',
    lng: '16.381373',
    rad: '10',
    tags:
     [ 'Italian',
       'Continental',
       'Austrian',
       'Mediterranean',
       'Vegan',
       'Vegetarian',
       'Steakhouses',
       'Steak',
       'Mittagessen' ],
    types:
     [ 'Italian',
       'Continental',
       'Austrian',
       'Mediterranean',
       'American' ],
    location_type_id: 'restaurant',
    createdAt: null,
    updatedAt: null,
    content: { content: [Object] },
    rating: '5',
    description: 'Das Urban´s der neue Hotspot - ein Restaurant bei dem Genuss und gesunde Ernährung kein Widerspruch sind. Entdecken Sie unsere aromatische und delikate Küche in kosmopolitischem Flair. Gustieren Sie in einem ungezwungenen Ambiente und lernen Sie unsere vielfältige internationale Küche kennen - vom gesunden Start in den Morgen, dem kleinen Lunch für zwischendurch oder den köstlichen Dinner-Variationen. Verweilen Sie im Anschluss in unserer stylischen Urban’s Lounge beim chilligen Afterwork-Drink. Raucher sind uns in unserem überdachten Gastgarten willkommen. Wir hoffen, auch Sie schon bald im Urban’s begrüßen zu dürfen. Stay tuned…',
    cover_image: null,
    local_timezone: 'Europe/Vienna',
    min_arrival_timespan_in_minutes: '30',
    max_reservation_capacity: 16,
    smoking_zone: false,
    language: 'de',
    allow_discount: true,
    allow_reservation: true,
    allow_pre_order: true }
 *
 * @param onFail (message)
 * @param onError (error)
 */
locationService.getLocationAndDiscounts = function (_id, _active, _pos, _length, _format, _lang, onSuccess, onFail, onError) {
    var query = '';

    if(_pos != null && _length != null && _active != null) {
        query = '?pos=' + _pos + '&length=' + _length + '&active=' + _active + '&format=' + _format + '&lang=' + _lang;
    }

    request.getJSON(null, config.api('/public/locations/discounts/' + _id + query),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.location, body.discounts);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this query search to get all nearby locations by lat/lng.
 *
 * @param onSuccess(locations) is called when locations where successfully found, distance and discounts are also attached
 * @param onFail (message) is called when searching for locations fails
 * @param onError (error) is called on error or if search for locations fails
 */
locationService.search = function (_lat, _lng, _accuracy, onSuccess, onFail, onError) {
    var query = '';

    if(_lat != null && _lng != null && _accuracy != null) {
        query = '?lat=' + _lat + '&lng=' + _lng + '&accuracy=' + _accuracy;
    }

    request.getJSON(null, config.api('/public/locations/query/search' + query),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.locations);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

module.exports = locationService;

