var config = require('./config');
var request = require('./request');

var menuService = {};

/**
 * Export the config for the env
 */
menuService.config = config;

/**
 * Menus
 */

/**
 * Use this method to get an menu by id.
 * @param _menu_id must not be null, must be a valid menu UUID
 * @param onSuccess(menu) is called on successfully retrieving menu
 * @param onFail(message) is called on failure
 * @param onError(error) is called on error
 */
menuService.getMenu = function (_menu_id, onSuccess, onFail, onError) {
    request.getJSON(null, config.api('/public/menus/' + _menu_id),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.menu);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to add a menu to a location.
 * @param _token must not be null, must be a valid client token
 * @param _location_id must not be null, must be a valid location id
 * @param _content
 * @param _title
 * @param _start_year
 * @param _start_month
 * @param _start_day
 * @param _end_year
 * @param _end_month
 * @param _end_day
 * @param _weekdays
 * @param onSuccess(menu) is called when the menu was successfully added
 * @param onFail(message) is called when adding the menu fails
 * @param onError(err) is called on error or if adding the menu fails
 */
menuService.addMenu = function (_token,
                                _location_id,
                                _content,
                                _title,
                                _start_year,
                                _start_month,
                                _start_day,
                                _end_year,
                                _end_month,
                                _end_day,
                                _weekdays,
                                onSuccess, onFail, onError) {
    if (_token == null) {
        return onError(new Error('Invalid token was given.'));
    }

    request.putJSON(_token,
        config.api('/menus'),
        {
            'location_id': _location_id,
            'content': _content,
            'title': _title,
            'start_year': _start_year,
            'start_month': _start_month,
            'start_day': _start_day,
            'end_year': _end_year,
            'end_month': _end_month,
            'end_day': _end_day,
            'weekdays': _weekdays
        },
        function (body) {
            if (onSuccess != null) {
                return onSuccess(body.menu);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to update an menu by id (UUID).
 * @param _token must not be null, must be a valid client token
 * @param _menu_id must not be null, must be a valid menu id (UUID)
 * @param _content
 * @param _title
 * @param _start_year
 * @param _start_month
 * @param _start_day
 * @param _end_year
 * @param _end_month
 * @param _end_day
 * @param _weekdays
 * @param onSuccess(menu) is called when the menu was successfully updated
 * @param onFail(message) is called when updating the menu fails
 * @param onError(err) is called on error or if updating the menu fails
 */
menuService.updateMenu = function (_token,
                                   _menu_id,
                                   _content,
                                   _title,
                                   _start_year,
                                   _start_month,
                                   _start_day,
                                   _end_year,
                                   _end_month,
                                   _end_day,
                                   _weekdays,
                                   onSuccess, onFail, onError) {
    if (_token == null) {
        return onError(new Error('Invalid token was given.'));
    }

    request.postJSON(_token,
        config.api('/menus/' + _menu_id),
        {
            'content': _content,
            'title': _title,
            'start_year': _start_year,
            'start_month': _start_month,
            'start_day': _start_day,
            'end_year': _end_year,
            'end_month': _end_month,
            'end_day': _end_day,
            'weekdays': _weekdays
        },
        function (body) {
            if (onSuccess != null) {
                return onSuccess(body.menu);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to set the menu status for a given menu
 * @param _token must not be null, must be valid client token, the client must be the owner of the menu
 * @param _menu_id must not be null, must be valid menu id
 * @param _status must not be null, must be a valid status: 1. draft 2. published
 * @param onSuccess(menu) is called when the menu was successfully updated
 * @param onFail(message) is called when updating the menu fails
 * @param onError(err) is called on error or if updating the menu fails
 */
menuService.setMenuStatus = function (_token, _menu_id, _status, onSuccess, onFail, onError) {
    if (_token == null) {
        return onError(new Error('Invalid token was given.'));
    }

    if(_status == null) {
        return onError('Invalid status was given.');
    }

    request.postJSON(_token,
        config.api('/menus/' + _menu_id + '/state?status=' + _status),
        {
            /* empty body */
        },
        function (body) {
            if (onSuccess != null) {
                return onSuccess(body.menu);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to get an complete menu card including categories and menuItems by menu id.
 * @param _menu_id must not be null, must be a valid menu UUID
 * @param onSuccess(menu) is called on successfully retrieving menu card
 * @param onFail(message) is called on failure
 * @param onError(error) is called on error
 */
menuService.getMenuCard = function (_menu_id, onSuccess, onFail, onError) {
    request.getJSON(null, config.api('/public/menus/card/' + _menu_id),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.menu);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to retrieve all menus from a given location.
 * @param _location_id must not be null, must be valid location id (UUID)
 * @param _pos may be null, default 0
 * @param _length may be null, default 30
 * @param onSuccess(menus) is called when retrieving the menus was successful
 * @param onFail(message) is called when retrieving the menus fails
 * @param onError(err) is called on error
 */
menuService.getMenusByLocation = function (_location_id, _pos, _length, onSuccess, onFail, onError) {
    var query = '';

    if(_pos != null && _length != null) {
        query = '?pos=' + _pos + '&length=' + _length;
    }
    
    request.getJSON(null, config.api('/public/menus/all/' + _location_id + query),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.menus);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Menu categories
 */

/**
 * Use this method to get an menu category by id (UUID).
 * @param _menu_category_id must not be null, must be a valid menu category UUID
 * @param onSuccess(category) is called on successfully retrieving menu
 * @param onFail(message) is called on failure
 * @param onError(error) is called on error
 */
menuService.getMenuCategory = function (_menu_category_id, onSuccess, onFail, onError) {
    request.getJSON(null, config.api('/public/menus/categories/' + _menu_category_id),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.category);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to add a category to an menu by id (UUID).
 * @param _token must not null, must be a valid client token
 * @param _menu_id must not be null, must be a valid menu id (UUID)
 * @param _title
 * @param _content
 * @param _start_hour
 * @param _start_min
 * @param _end_hour
 * @param _end_min
 * @param _tags
 * @param _images
 * @param _front_image
 * @param _weekdays
 * @param onSuccess(category) is called when the category was successfully added to the menu
 * @param onFail(message) is called when adding the category fails
 * @param onError(err) is called on error or if adding the category fails
 */
menuService.addMenuCategory = function (_token,
                                        _menu_id,
                                        _title,
                                        _content,
                                        _start_hour,
                                        _start_min,
                                        _end_hour,
                                        _end_min,
                                        _tags,
                                        _images,
                                        _front_image,
                                        _weekdays,
                                        onSuccess, onFail, onError) {
    if (_token == null) {
        return onError(new Error('Invalid token was given.'));
    }

    request.putJSON(_token,
        config.api('/menus/' + _menu_id + '/categories'),
        {
            'title': _title,
            'content': _content,
            'start_hour': _start_hour,
            'start_min': _start_min,
            'end_hour': _end_hour,
            'end_min': _end_min,
            'tags': _tags,
            'images': _images,
            'front_image': _front_image,
            'weekdays': _weekdays
        },
        function (body) {
            if (onSuccess != null) {
                return onSuccess(body.category);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to update an category by id (UUID).
 * @param _token must not null, must be a valid client token
 * @param _menu_id must not be null, must be a valid menu id (UUID)
 * @param _title
 * @param _content
 * @param _start_hour
 * @param _start_min
 * @param _end_hour
 * @param _end_min
 * @param _tags
 * @param _images
 * @param _front_image
 * @param _weekdays
 * @param onSuccess(category) is called when the category was successfully added to the menu
 * @param onFail(message) is called when adding the category fails
 * @param onError(err) is called on error or if adding the category fails
 */
menuService.updateMenuCategory = function (_token,
                                           _menu_id,
                                           _category_id,
                                           _title,
                                           _content,
                                           _start_hour,
                                           _start_min,
                                           _end_hour,
                                           _end_min,
                                           _tags,
                                           _images,
                                           _front_image,
                                           _weekdays,
                                           onSuccess, onFail, onError) {
    if (_token == null) {
        return onError(new Error('Invalid token was given.'));
    }

    request.postJSON(_token,
        config.api('/menus/' + _menu_id + '/categories/' + _category_id),
        {
            'title': _title,
            'content': _content,
            'start_hour': _start_hour,
            'start_min': _start_min,
            'end_hour': _end_hour,
            'end_min': _end_min,
            'tags': _tags,
            'images': _images,
            'front_image': _front_image,
            'weekdays': _weekdays
        },
        function (body) {
            if (onSuccess != null) {
                return onSuccess(body.category);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to get an all menu categories by an menu id (UUID).
 * @param _menu_id must not be null, must be a valid menu UUID
 * @param onSuccess(categories) is called on successfully retrieving categories, an json array of menu category
 * @param onFail(message) is called on failure
 * @param onError(error) is called on error
 */
menuService.getMenuCategories = function (_menu_id, onSuccess, onFail, onError) {
    request.getJSON(null, config.api('/public/menus/' + _menu_id + '/categories/all'),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.categories);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Menu items
 */

/**
 * Use this method to get an menu item by id (UUID).
 * @param _menu_item_id must not be null, must be a valid menu item UUID
 * @param onSuccess(item) is called on successfully retrieving menu item
 * @param onFail(message) is called on failure
 * @param onError(error) is called on error
 */
menuService.getMenuItem = function (_menu_item_id, onSuccess, onFail, onError) {
    request.getJSON(null, config.api('/public/menus/items/' + _menu_item_id),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.item);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to get all selection menu items from an menu item
 * @param _menu_item_id must not be null, must be a valid menu item UUID
 * @param onSuccess(item, selection) is called on successfully retrieving the selection menu items
 * @param onFail(message) is called on failure
 * @param onError(error) is called on error
 */
menuService.getSelectionMenuItems = function (_menu_item_id, onSuccess, onFail, onError) {
    request.getJSON(null, config.api('/public/menus/items/' + _menu_item_id + '/selection'),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.item, body.selection);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to add an menu item to an menu category by id (UUID).
 * @param _token must not be null, must be valid client token
 * @param _category_id must not be null, must be valid category id, where the menu item will be added
 * @param _content must not be null, may be an empty json {}
 * @param _title must not be null, must be a json containing the title of the item separated by the language, like this: { "en": "My menuItem", "de": "Meine speise" }
 * @param _price must not be null, must be a valid price as integer, no fraction are allowed, must be passed as an lowest valute, for example: 7.90 EUR => 790 cents
 * @param _currency must not be null, must be a valid currency
 * @param _description may be null
 * @param _calories may be null, must be numeric if passed
 * @param _fats may be null, must be numeric if passed
 * @param _proteins may be null, must be numeric if passed
 * @param _carbohydrates may be null, must be numeric if passed
 * @param _tastes may be null, must be an array of tastes if passed
 * @param _cuisine may be null, must be an array of cusine if passed
 * @param _beverage_preferences may be null, must be an array of beverage prefs if passed
 * @param _food_preferences may be null, must be an array of food prefs if passed
 * @param _allergies may be null, must be an array of allergies if passed
 * @param _tags may be null, must be an array of tags if passed
 * @param _images may be null, must be an array of UUIDs if passed
 * @param _front_image may be null
 * @param _is_menu may not be null, If this field is set to true, then you must supply optional menu items in the optional_items. Optional items are menuItems that are served within this item. With this field you can build daily menus. The given price is the currents(this) price item.
 * @param _optional_items empty array or must be an array of UUIDs (other menuItems that will be combined with this menuItem) if passed
 * @param onSuccess(menuItem) is called when the menu item was successfully added to the category
 * @param onFail(message) is called when adding the menu item fails
 * @param onError(err) is called on error or if adding the menu item fails
  */
menuService.addMenuItem = function (_token,
                                    _category_id,
                                    _content,
                                    _title,
                                    _price,
                                    _currency,
                                    _description,
                                    _calories,
                                    _fats,
                                    _proteins,
                                    _carbohydrates,
                                    _tastes,
                                    _cuisine,
                                    _beverage_preferences,
                                    _food_preferences,
                                    _allergies,
                                    _tags,
                                    _images,
                                    _front_image,
                                    _is_menu,
                                    _optional_items,
                                    _selection_items, onSuccess, onFail, onError) {
    if (_token == null) {
        return onError(new Error('Invalid token was given.'));
    }

    request.putJSON(_token,
        config.api('/menus/categories/' + _category_id),
        {
            'content': _content,
            'title': _title,
            'price': _price,
            'currency': _currency,
            'description': _description,
            'calories': _calories,
            'fats': _fats,
            'proteins': _proteins,
            'carbohydrates': _carbohydrates,
            'tastes': _tastes,
            'cuisine': _cuisine,
            'beverage_preferences': _beverage_preferences,
            'food_preferences': _food_preferences,
            'allergies': _allergies,
            'tags': _tags,
            'images': _images,
            'front_image': _front_image,
            'is_menu': _is_menu,
            'optional_items': _optional_items,
            'selection_items': _selection_items
        },
        function (body) {
            if (onSuccess != null) {
                return onSuccess(body.item);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to update an menu item to an menu category by id (UUID).
 * @param _token must not be null, must be valid client token
 * @param _category_id must not be null, must be valid category id, where the menu item will be added
 * @param _content must not be null, may be an empty json {}
 * @param _title must not be null, must be a json containing the title of the item separated by the language, like this: { "en": "My menuItem", "de": "Meine speise" }
 * @param _price must not be null, must be a valid price as integer, no fraction are allowed, must be passed as an lowest valute, for example: 7.90 EUR => 790 cents
 * @param _currency must not be null, must be a valid currency
 * @param _description may be null
 * @param _calories may be null, must be numeric if passed
 * @param _fats may be null, must be numeric if passed
 * @param _proteins may be null, must be numeric if passed
 * @param _carbohydrates may be null, must be numeric if passed
 * @param _tastes may be null, must be an array of tastes if passed
 * @param _cuisine may be null, must be an array of cusine if passed
 * @param _beverage_preferences may be null, must be an array of beverage prefs if passed
 * @param _food_preferences may be null, must be an array of food prefs if passed
 * @param _allergies may be null, must be an array of allergies if passed
 * @param _tags may be null, must be an array of tags if passed
 * @param _images may be null, must be an array of UUIDs if passed
 * @param _front_image may be null
 * @param _is_menu may not be null, If this field is set to true, then you must supply optional menu items in the optional_items. Optional items are menuItems that are served within this item. With this field you can build daily menus. The given price is the currents(this) price item.
 * @param _optional_items empty array or must be an array of UUIDs (other menuItems that will be combined with this menuItem) if passed
 * @param onSuccess(menuItem) is called when the menu item was successfully updated to the category
 * @param onFail(message) is called when updating the menu item fails
 * @param onError(err) is called on error or if updating the menu item fails
 */
menuService.updateMenuItem = function (_token,
                                    _menu_item_id,
                                    _category_id,
                                    _content,
                                    _title,
                                    _price,
                                    _currency,
                                    _description,
                                    _calories,
                                    _fats,
                                    _proteins,
                                    _carbohydrates,
                                    _tastes,
                                    _cuisine,
                                    _beverage_preferences,
                                    _food_preferences,
                                    _allergies,
                                    _tags,
                                    _images,
                                    _front_image,
                                    _is_menu,
                                    _optional_items,
                                    _selection_items, onSuccess, onFail, onError) {
    if (_token == null) {
        return onError(new Error('Invalid token was given.'));
    }

    request.putJSON(_token,
        config.api('/menus/categories/' + _category_id + '/items/' + _menu_item_id),
        {
            'content': _content,
            'title': _title,
            'price': _price,
            'currency': _currency,
            'description': _description,
            'calories': _calories,
            'fats': _fats,
            'proteins': _proteins,
            'carbohydrates': _carbohydrates,
            'tastes': _tastes,
            'cuisine': _cuisine,
            'beverage_preferences': _beverage_preferences,
            'food_preferences': _food_preferences,
            'allergies': _allergies,
            'tags': _tags,
            'images': _images,
            'front_image': _front_image,
            'is_menu': _is_menu,
            'optional_items': _optional_items,
            'selection_items': _selection_items
        },
        function (body) {
            if (onSuccess != null) {
                return onSuccess(body.old, body.item);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

module.exports = menuService;

