var config = require('./config');
var request = require('./request');

var reminderService = {};

/**
 * Export the config for the env
 */
reminderService.config = config;

/**
 * Use this method to set a new reminder.
 * @param _key must not be null, must be a unique key
 * @param _ms must not be null, must be > 0, the timeout of the reminder in milliseconds
 * @param _type must not be null, the type of request to execute after the timeout, may be: put, post, get
 * @param _url must not be null, must be a valid url, the url to execute the request after the timeout
 * @param _payload may be null, optional body payload for the request to be executed after the timeout
 * @param onSuccess() is called when setting the timer for successful
 * @param onFail(message) is called on fail or if setting the timer fails
 * @param onError(err) is called on error or if setting the timer fails
 */
reminderService.setReminder = function (_key, _ms, _type, _url, _payload, onSuccess, onFail, onError) {
    if(_key == null) {
        return onError(new Error('You must provide a unique key for the reminder.'));
    }

    if(_ms == null) {
        return onError(new Error('You must provide a the timeout for the reminder.'));
    }

    request.putJSON(null,
        config.apiReminder('/'),
        {
            key: _key,
            ms: _ms,
            type: _type,
            url: _url,
            payload: _payload
        },
        function (body) {
            if(onSuccess != null) {
                return onSuccess();
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to set batch reminders.
 * The batch reminder must be an array of the following type:
 *
 * [
 *   {
 *           key: _key,
 *           ms: _ms,
 *           type: _type,
 *           url: _url,
 *           payload: _payload
 *   }
 * ]
 *
 * @param _batch must not be null, must not be empty
 * @param onSuccess() is called when the batch reminder were successfully set
 * @param onFail() is never called
 * @param onError(err) is called on error. Attention: This error callback can be executed multiple times, make sure that
 * yo do NOT execute synchronous or data dependet code
 */
reminderService.setBatchReminders = function (_batch, onSuccess, onFail, onError) {
    if(_batch != null && Array.isArray(_batch) && _batch.length > 0) {
        for(var i = 0; i < _batch.length; i++) {
            var elem = _batch[i];

            if(elem.key && elem.ms && elem.type && elem.url && elem.payload) {
                reminderService.setReminder(elem.key, elem.ms, elem.type, elem.url, elem.payload,
                    function () { /* do nothing */ }, function (message) {
                    console.error(message);
                }, function (err) {
                    console.error(err);
                });
            } else {
                /**
                 * Do not stop the for at an invalid elem
                 */
               onError(new Error('Batch element at index' + i + ' has invalid args. You must provide an key, ms, type, url and payload'));
            }
        }

        return onSuccess();
    } else {
        return onError(new Error('Invalid _batch arg was given (null or empty).'));
    }
};

/**
 * Use this method to delete an active reminder
 * @param _key must not be null, must be a unique key
  * @param onSuccess() is called when deleting the active reminder was successful
 * @param onFail(message) is called on fail or if deleting the reminder fails
 * @param onError(err) is called on error or if deleting the reminder fails
 */
reminderService.deleteReminder = function (_key, onSuccess, onFail, onError) {
    if(_key == null) {
        return onError(new Error('You must provide a unique key for the reminder.'));
    }

    request.post(null,
        config.apiReminder('/remove/' + _key),
        { /* empty body */ },
        function (body) {
            if(onSuccess != null) {
                return onSuccess();
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to check if an active reminder for the given key is set and NOT timed out
 * @param _key must not be null, must be a unique key
 * @param onSuccess() is called when a running reminder is available for the given key.
 * @param onFail(message) is called when no active reminder is set
 * @param onError(err) is called on error
 */
reminderService.isSet = function(_key, onSuccess, onFail, onError) {
    if(_key == null) {
        return onError(new Error('You must provide a unique key for the reminder.'));
    }

    request.getJSON(null,
        config.apiReminder('/' + _key),
        function (body) {
            if(onSuccess != null) {
                return onSuccess();
            }
        },
        function (message) {
            return onFail();
        },
        function (err) {
            return onError(err);
        });
};

module.exports = reminderService;
