var assert = require('assert');
var chai = require('chai');
var expect = chai.expect; // we are using the "expect" style of Chai

var testService = require('./test');
var authService = require('./../auth');
var clientService = require('./../client');
var menuService = require('./../menu');
var basketService = require('./../basket');

var client = 'client@test.com'
var password = '123';
var token = '';

var locationId = null;
var menus = null;
var menu = null;
var card = null;
var category = null;

describe('Basket & Menus', function () {
    before(function (done) {
        /**
         * Migrate the test data
         */
        testService.config.setEnvironment('localhost');
        testService.migrate(function (message) {
            console.log(message);
            setup(done);
        }, function (message) {
            console.error(message);
            return done(new Error(message));
        }, function (err) {
            console.error(err);
            return done(err);
        });
    });

    after(function (done) {
        /**
         * Drop the test data
         */
        testService.truncate(function (message) {
            console.log(message);
            done();
        }, function (message) {
            console.error(message);
            done(new Error(message));
        }, function (err) {
            console.error(err);
            done(err);
        });
    });

    /**
     * Init. the testing environment
     */
    function setup(done) {
        /**
         * set the api environment to localhost for the CI setup
         */
        authService.config.setEnvironment('localhost');
        menuService.config.setEnvironment('localhost');
        clientService.config.setEnvironment('localhost');
        basketService.config.setEnvironment('localhost');

        /**
         * auth. this test/account
         */
        authService.authenticate(client, password, function (_token, _account) {
            token = _token;

            /**
             * Get the location id
             */
            clientService.getClientsLocations(token, function (locations) {
                if (locations.length > 0) {
                    locationId = locations[0].id;

                    /**
                     * Get the menus, card, category from the location
                     */
                    menuService.getMenusByLocation(locationId, null, null, function (m) {
                        if (Array.isArray(m) && m.length > 0) {
                            menus = m;
                            menu = m[0];

                            menuService.getMenuCard(menu.id, function (c) {
                                card = c;

                                if (Array.isArray(card.categories) && card.categories.length > 0) {
                                    category = card.categories[0];
                                    done();
                                } else {
                                    done(new Error('No categories were found to run against the test suit.'));
                                }
                            }, function (message) {
                                done(new Error(message));
                            }, function (err) {
                                done(err);
                            });
                        } else {
                            done(new Error('No menus were found to run against the test suit.'));
                        }
                    }, function (message) {
                        done(new Error(message))
                    }, function (err) {
                        done(err);
                    });
                } else {
                    done(new Error("No location was found to run the tests against."));
                }
            }, function (message) {
                done(new Error(message));
            }, function (err) {
                done(err);
            });
        }, function (message) {
            console.error('Account auth. failed, test suit aborted.')
            done(new Error(message));
        }, function (err) {
            console.error('Account auth. failed, test suit aborted.')
            done(err);
        });
    }

    /**
     * Test the MenuService
     */
    describe('BasketService', function () {
        describe('Insert', function () {
            describe('addMenuItem(_menu_item_id, _selection_items_, onSuccess, onFail, onError))', function () {
                it('should successfully add a menuItem to the basket', function (done) {
                    menuService.addMenuItem(token, category.id, {}, {
                            "de": "Mini Burger",
                            "en": "Mini Burger"
                        }, 1090, 'EUR', {
                            "en": "The best burger in Town",
                            "de": "Bester Burger in Wien"
                        }, 1000, 40, 35, 60, null, null, null, null, null, {
                            "en": ["Burger"],
                            "de": []
                        }, null, null, false, [], null,
                        function (menuItem) {
                            var basket = basketService.instance();

                            /**
                             * Asset that the basket is empty
                             */
                            assert.equal(basket.totalPrice, 0);
                            assert.equal(basket.quantity(), 0);
                            assert.equal(basket.size(), 0);

                            basket.addMenuItem(menuItem.id, null,
                                function (_item) {
                                    /**
                                     * Asset basic checks that the given item was added
                                     */
                                    assert.equal(basket.size(), 1);
                                    assert.equal(basket.quantity(), 1);
                                    assert.equal(_item.id, menuItem.id);
                                    assert.equal(basket.items[0].item.id, menuItem.id);
                                    assert.equal(basket.items[0].item.price, menuItem.price);
                                    assert.equal(basket.items[0].quantity, 1);
                                    assert.equal(basket.totalPrice, menuItem.price);
                                    assert.equal(basket.totalPrice, basket.items[0].item.price * basket.items[0].quantity);

                                    done();
                                }, function (message) {
                                    done(new Error(message));
                                }, function (err) {
                                    done(err);
                                });
                        }, function (message) {
                            done(new Error(message));
                        }, function (err) {
                            done(err);
                        });
                });
            });

            describe('addMenuItem(_menu_item_id, _selection_items_, onSuccess, onFail, onError))', function () {
                it('should successfully add multiple times of the same menuItem to the basket', function (done) {
                    menuService.addMenuItem(token, category.id, {}, {
                            "de": "Mini Burger",
                            "en": "Mini Burger"
                        }, 1090, 'EUR', {
                            "en": "The best burger in Town",
                            "de": "Bester Burger in Wien"
                        }, 1000, 40, 35, 60, null, null, null, null, null, {
                            "en": ["Burger"],
                            "de": []
                        }, null, null, false, [], null,
                        function (menuItem) {
                            var basket = basketService.instance();

                            /**
                             * Asset that the basket is empty
                             */
                            assert.equal(basket.totalPrice, 0);
                            assert.equal(basket.quantity(), 0);
                            assert.equal(basket.size(), 0);

                            basket.addMenuItem(menuItem.id, null,
                                function () {
                                    basket.addMenuItem(menuItem.id, null,
                                        function () {
                                            basket.addMenuItem(menuItem.id, null,
                                                function (_item) {
                                                    /**
                                                     * Asset basic checks that the given item was added
                                                     */

                                                    /**
                                                     * Check sizing of the basket
                                                     */
                                                    assert.equal(basket.size(), 1);
                                                    assert.equal(basket.quantity(), 3);

                                                    /**
                                                     * verificate menuItem
                                                     */
                                                    assert.equal(_item.id, menuItem.id);
                                                    assert.equal(basket.items.length, 1);
                                                    assert.equal(basket.items[0].item.id, menuItem.id);
                                                    assert.equal(basket.items[0].item.price, menuItem.price);
                                                    assert.equal(basket.items[0].quantity, 3);
                                                    assert.equal(basket.totalPrice, basket.items[0].item.price * basket.items[0].quantity);

                                                    done();
                                                }, function (message) {
                                                    done(new Error(message));
                                                }, function (err) {
                                                    done(err);
                                                });
                                        }, function (message) {
                                            done(new Error(message));
                                        }, function (err) {
                                            done(err);
                                        });
                                }, function (message) {
                                    done(new Error(message));
                                }, function (err) {
                                    done(err);
                                });
                        }, function (message) {
                            done(new Error(message));
                        }, function (err) {
                            done(err);
                        });
                });
            });

            describe('addMenuItem(_menu_item_id, _selection_items_, onSuccess, onFail, onError))', function () {
                it('should successfully add multiple different menuItems to the basket', function (done) {
                    menuService.addMenuItem(token, category.id, {}, {
                            "de": "Mini Burger",
                            "en": "Mini Burger"
                        }, 1090, 'EUR', {
                            "en": "The best burger in Town",
                            "de": "Bester Burger in Wien"
                        }, 1000, 40, 35, 60, null, null, null, null, null, {
                            "en": ["Burger"],
                            "de": []
                        }, null, null, false, [], null,
                        function (menuItem1) {
                            menuService.addMenuItem(token, category.id, {}, {
                                    "de": "Mini Burger",
                                    "en": "Mini Burger"
                                }, 1090, 'EUR', {
                                    "en": "The best burger in Town",
                                    "de": "Bester Burger in Wien"
                                }, 1000, 40, 35, 60, null, null, null, null, null, {
                                    "en": ["Burger"],
                                    "de": []
                                }, null, null, false, [], null,
                                function (menuItem2) {
                                    var basket = basketService.instance();

                                    /**
                                     * Asset that the basket is empty
                                     */
                                    assert.equal(basket.totalPrice, 0);
                                    assert.equal(basket.quantity(), 0);
                                    assert.equal(basket.size(), 0);

                                    basket.addMenuItem(menuItem1.id, null,
                                        function (_item1) {
                                            basket.addMenuItem(menuItem2.id, null,
                                                function (_item2) {
                                                    /**
                                                     * Asset basic checks that the given item was added
                                                     */
                                                    assert.equal(basket.size(), 2);
                                                    assert.equal(basket.quantity(), 2);
                                                    assert.equal(_item1.id, menuItem1.id);
                                                    assert.equal(_item2.id, menuItem2.id);
                                                    assert.equal(basket.items[0].item.id, menuItem1.id);
                                                    assert.equal(basket.items[1].item.id, menuItem2.id);
                                                    assert.equal(basket.items[0].item.price, menuItem1.price);
                                                    assert.equal(basket.items[1].item.price, menuItem2.price);
                                                    assert.equal(basket.items[0].quantity, 1);
                                                    assert.equal(basket.items[1].quantity, 1);
                                                    assert.equal(basket.totalPrice, (basket.items[0].item.price * basket.items[0].quantity) + (basket.items[1].item.price * basket.items[1].quantity));

                                                    done();
                                                }, function (message) {
                                                    done(new Error(message));
                                                }, function (err) {
                                                    done(err);
                                                });
                                        }, function (message) {
                                            done(new Error(message));
                                        }, function (err) {
                                            done(err);
                                        });
                                }, function (message) {
                                    done(new Error(message));
                                }, function (err) {
                                    done(err);
                                });
                        }, function (message) {
                            done(new Error(message));
                        }, function (err) {
                            done(err);
                        });
                });
            });
        });

        describe('Remove', function () {
            describe('removeMenuItem(_menu_item_id, _selection_items_, onSuccess, onFail, onError))', function () {
                it('should successfully add and remove a menuItem from the basket', function (done) {
                    menuService.addMenuItem(token, category.id, {}, {
                            "de": "Mini Burger",
                            "en": "Mini Burger"
                        }, 1090, 'EUR', {
                            "en": "The best burger in Town",
                            "de": "Bester Burger in Wien"
                        }, 1000, 40, 35, 60, null, null, null, null, null, {
                            "en": ["Burger"],
                            "de": []
                        }, null, null, false, [], null,
                        function (menuItem) {
                            var basket = basketService.instance();

                            /**
                             * Asset that the basket is empty
                             */
                            assert.equal(basket.totalPrice, 0);
                            assert.equal(basket.quantity(), 0);
                            assert.equal(basket.size(), 0);

                            basket.addMenuItem(menuItem.id, null,
                                function (_item) {
                                    /**
                                     * Asset basic checks that the given item was added
                                     */
                                    assert.equal(basket.size(), 1);
                                    assert.equal(basket.quantity(), 1);
                                    assert.equal(_item.id, menuItem.id);
                                    assert.equal(basket.items[0].item.id, menuItem.id);
                                    assert.equal(basket.items[0].item.price, menuItem.price);
                                    assert.equal(basket.items[0].quantity, 1);
                                    assert.equal(basket.totalPrice, menuItem.price);
                                    assert.equal(basket.totalPrice, basket.items[0].item.price * basket.items[0].quantity);

                                    basket.removeMenuItem(menuItem.id, null, function (removedItem) {
                                        /**
                                         * Asset that the basket is empty
                                         */
                                        assert.equal(basket.totalPrice, 0);
                                        assert.equal(basket.quantity(), 0);
                                        assert.equal(basket.size(), 0);
                                        assert.equal(removedItem.id, menuItem.id);
                                        assert.equal(basket.items.length, 0);

                                        done();
                                    }, function (message) {
                                        done(new Error(message));
                                    }, function (err) {
                                        done(err);
                                    });
                                }, function (message) {
                                    done(new Error(message));
                                }, function (err) {
                                    done(err);
                                });
                        }, function (message) {
                            done(new Error(message));
                        }, function (err) {
                            done(err);
                        });
                });
            });

            describe('removeMenuItem(_menu_item_id, _selection_items_, onSuccess, onFail, onError))', function () {
                it('should fail to remove menu item from the basket due to invalid menu item id', function (done) {
                    menuService.addMenuItem(token, category.id, {}, {
                            "de": "Mini Burger",
                            "en": "Mini Burger"
                        }, 1090, 'EUR', {
                            "en": "The best burger in Town",
                            "de": "Bester Burger in Wien"
                        }, 1000, 40, 35, 60, null, null, null, null, null, {
                            "en": ["Burger"],
                            "de": []
                        }, null, null, false, [], null,
                        function (menuItem) {
                            var basket = basketService.instance();

                            /**
                             * Asset that the basket is empty
                             */
                            assert.equal(basket.totalPrice, 0);
                            assert.equal(basket.quantity(), 0);
                            assert.equal(basket.size(), 0);

                            basket.addMenuItem(menuItem.id, null,
                                function (_item) {
                                    /**
                                     * Asset basic checks that the given item was added
                                     */
                                    assert.equal(basket.size(), 1);
                                    assert.equal(basket.quantity(), 1);
                                    assert.equal(_item.id, menuItem.id);
                                    assert.equal(basket.items[0].item.id, menuItem.id);
                                    assert.equal(basket.items[0].item.price, menuItem.price);
                                    assert.equal(basket.items[0].quantity, 1);
                                    assert.equal(basket.totalPrice, menuItem.price);
                                    assert.equal(basket.totalPrice, basket.items[0].item.price * basket.items[0].quantity);

                                    basket.removeMenuItem('test', null, function (removedItem) {
                                        /**
                                         * Asset that the basket is empty
                                         */
                                        assert.equal(basket.totalPrice, 0);
                                        assert.equal(basket.quantity(), 0);
                                        assert.equal(basket.size(), 0);
                                        assert.equal(removedItem.id, menuItem.id);
                                        assert.equal(basket.items.length, 0);

                                        done(new Error('Should fail to remove the menu item.'));
                                    }, function (message) {
                                        done();
                                    }, function (err) {
                                        done();
                                    });
                                }, function (message) {
                                    done(new Error(message));
                                }, function (err) {
                                    done(err);
                                });
                        }, function (message) {
                            done(new Error(message));
                        }, function (err) {
                            done(err);
                        });
                });
            });

            describe('removeMenuItem(_menu_item_id, _selection_items_, onSuccess, onFail, onError))', function () {
                it('should successfully add multiple different menuItems and remove one menuItem from the basket', function (done) {
                    menuService.addMenuItem(token, category.id, {}, {
                            "de": "Mini Burger",
                            "en": "Mini Burger"
                        }, 1090, 'EUR', {
                            "en": "The best burger in Town",
                            "de": "Bester Burger in Wien"
                        }, 1000, 40, 35, 60, null, null, null, null, null, {
                            "en": ["Burger"],
                            "de": []
                        }, null, null, false, [], null,
                        function (menuItem1) {
                            menuService.addMenuItem(token, category.id, {}, {
                                    "de": "Mini Burger",
                                    "en": "Mini Burger"
                                }, 1090, 'EUR', {
                                    "en": "The best burger in Town",
                                    "de": "Bester Burger in Wien"
                                }, 1000, 40, 35, 60, null, null, null, null, null, {
                                    "en": ["Burger"],
                                    "de": []
                                }, null, null, false, [], null,
                                function (menuItem2) {
                                    var basket = basketService.instance();

                                    /**
                                     * Asset that the basket is empty
                                     */
                                    assert.equal(basket.totalPrice, 0);
                                    assert.equal(basket.quantity(), 0);
                                    assert.equal(basket.size(), 0);

                                    basket.addMenuItem(menuItem1.id, null,
                                        function (_item1) {
                                            basket.addMenuItem(menuItem2.id, null,
                                                function (_item2) {
                                                    /**
                                                     * Asset basic checks that the given item was added
                                                     */
                                                    assert.equal(basket.size(), 2);
                                                    assert.equal(basket.quantity(), 2);
                                                    assert.equal(_item1.id, menuItem1.id);
                                                    assert.equal(_item2.id, menuItem2.id);
                                                    assert.equal(basket.items[0].item.id, menuItem1.id);
                                                    assert.equal(basket.items[1].item.id, menuItem2.id);
                                                    assert.equal(basket.items[0].item.price, menuItem1.price);
                                                    assert.equal(basket.items[1].item.price, menuItem2.price);
                                                    assert.equal(basket.items[0].quantity, 1);
                                                    assert.equal(basket.items[1].quantity, 1);
                                                    assert.equal(basket.totalPrice, (basket.items[0].item.price * basket.items[0].quantity) + (basket.items[1].item.price * basket.items[1].quantity));

                                                    basket.removeMenuItem(menuItem1.id, null,
                                                        function (removedItem) {
                                                            /**
                                                             * Asset that the basket is empty
                                                             */
                                                            assert.equal(basket.totalPrice, 1090);
                                                            assert.equal(basket.quantity(), 1);
                                                            assert.equal(basket.size(), 1);
                                                            assert.equal(removedItem.id, menuItem1.id);
                                                            assert.equal(basket.items.length, 1);

                                                            done();
                                                        }, function (message) {
                                                            done(new Error(message));
                                                        }, function (err) {
                                                            done(err);
                                                        });
                                                }, function (message) {
                                                    done(new Error(message));
                                                }, function (err) {
                                                    done(err);
                                                });
                                        }, function (message) {
                                            done(new Error(message));
                                        }, function (err) {
                                            done(err);
                                        });
                                }, function (message) {
                                    done(new Error(message));
                                }, function (err) {
                                    done(err);
                                });
                        }, function (message) {
                            done(new Error(message));
                        }, function (err) {
                            done(err);
                        });
                });
            });
        });

        describe('Format', function () {
            describe('removeMenuItem(_menu_item_id, _selection_items_, onSuccess, onFail, onError))', function () {
                it('should successfully add and remove a menuItem from the basket', function (done) {
                    assert.equal('100,00 €', basketService.formatPrice(10000, 2, "€", ".", ","));
                    assert.equal('100,10 €', basketService.formatPrice(10010, 2, "€", ".", ","));
                    assert.equal('0,01 €', basketService.formatPrice(1, 2, "€", ".", ","));
                    assert.equal('0,10 €', basketService.formatPrice(10, 2, "€", ".", ","));
                    done();
                });
            });

        });

        describe('Types & State', function () {
            describe('toJson()', function () {
                it('should test the state of the basket without any items', function (done) {
                    var basket = basketService.instance();

                    basket.setOrderType(basketService.reservationType());
                    assert.equal('reservation', basket.getOrderType());

                    basket.setFirstName('Max');
                    basket.setLastName('Muster');
                    basket.setMail('max@muster.com');
                    basket.setTelNumber('111111');
                    basket.setNumberOfPeople(3);
                    basket.setDay(11);
                    basket.setMonth(12);
                    basket.setYear(2017);
                    basket.setHours(11);
                    basket.setMinutes(0);
                    basket.setNotes('I am really hungry!');

                    var param = basket.toJSON();

                    assert.equal('Max', param.first_name);
                    assert.equal('Muster', param.last_name);
                    assert.equal('max@muster.com', param.mail);
                    assert.equal('111111', param.tel_number);
                    assert.equal('3', param.number_of_people);
                    assert.equal('11', param.day);
                    assert.equal('12', param.month);
                    assert.equal('2017', param.year);
                    assert.equal('11', param.hour);
                    assert.equal('0', param.min);
                    assert.equal('I am really hungry!', param.notes);

                    done();
                });
            });

            describe('reservationType()', function () {
                it('should return the reservation type', function (done) {
                    var basket = basketService.instance();

                    basket.setOrderType(basketService.reservationType());
                    assert.equal('reservation', basket.getOrderType());

                    basket.setOrderType(basketService.takeAwayType());
                    assert.equal('takeAway', basket.getOrderType());

                    basket.setOrderType(basketService.discountType());
                    assert.equal('discountOnly', basket.getOrderType());

                    done();
                });
            });

            describe('reservationType()', function () {
                it('should return the reservation type', function (done) {
                    assert.equal('reservation', basketService.reservationType());
                    done();
                });
            });

            describe('takeAway()', function () {
                it('should return the take away type', function (done) {
                    assert.equal('takeAway', basketService.takeAwayType());
                    done();
                });
            });

            describe('discountType()', function () {
                it('should return the discount type', function (done) {
                    assert.equal('discountOnly', basketService.discountType());
                    done();
                });
            });
        });
    });
});
