var assert = require('assert');
var chai = require('chai');
var expect = chai.expect; // we are using the "expect" style of Chai
var momentTz = require('moment-timezone');
var moment = require('moment');

var testService = require('./test');

var authService = require('./../auth');
var clientService = require('./../client');
var menuService = require('./../menu');
var discountService = require('./../discount');

var client = 'client@test.com'
var clientId = '';
var password = '123';
var token = '';

var locationId = null;
var globalLocation;
var menus = null;
var menu = null;
var card = null;
var category = null;

describe('Discounts', function() {
    before(function(done) {
        /**
         * Migrate the test data
         */
        testService.config.setEnvironment('localhost');
        testService.migrate(function (message) {
            console.log(message);
            setup(done);
        }, function (message) {
            console.error(message);
            return done(new Error(message));
        }, function (err) {
            console.error(err);
            return done(err);
        });
    });

    after(function(done) {
        /**
         * Drop the test data
         */
        testService.truncate(function (message) {
            console.log(message);
            done();
        }, function (message) {
            console.error(message);
            done(new Error(message));
        }, function (err) {
            console.error(err);
            done(err);
        });
    });

    /**
     * Init. the testing environment
     */
    function setup(done) {
        /**
         * set the api enviroment to localhost for the CI setup
         */
        authService.config.setEnvironment('localhost');
        menuService.config.setEnvironment('localhost');
        clientService.config.setEnvironment('localhost');
        discountService.config.setEnvironment('localhost');

        /**
         * auth. this test/account
         */
        authService.authenticate(client, password, function (_token, _account) {
            token = _token;
            clientId = _account.id;

            /**
             * Get the location id
             */
            clientService.getClientsLocations(token, function (locations) {
                if (locations.length > 0) {
                    locationId = locations[0].id;
                    globalLocation = locations[0];

                    /**
                     * Get the menus, card, category from the location
                     */
                    menuService.getMenusByLocation(locationId, null, null, function (m) {
                        if (Array.isArray(m) && m.length > 0) {
                            menus = m;
                            menu = m[0];

                            menuService.getMenuCard(menu.id, function (c) {
                                card = c;

                                if (Array.isArray(card.categories) && card.categories.length > 0) {
                                    category = card.categories[0];
                                    done();
                                } else {
                                    done(new Error('No categories were found to run against the test suit.'));
                                }
                            }, function (message) {
                                done(new Error(message));
                            }, function (err) {
                                done(err);
                            });
                        } else {
                            done(new Error('No menus were found to run against the test suit.'));
                        }
                    }, function (message) {
                        done(new Error(message))
                    }, function (err) {
                        done(err);
                    });
                } else {
                    done(new Error("No location was found to run the tests against."));
                }
            }, function (message) {
                done(new Error(message));
            }, function (err) {
                done(err);
            });
        }, function (message) {
            console.error('Account auth. failed, test suit aborted.')
            done(new Error(message));
        }, function (err) {
            console.error('Account auth. failed, test suit aborted.')
            done(err);
        });
    }

    /**
     * Test the userService and ordering
     */
    describe('DiscountService', function () {
        describe('#addDiscount(_token, _location_id, _discount, _items, _excluded_items, _start_year, _start_month, _start_day_of_month, _start_hour, _start_min, _end_year, _end_month, _end_day_of_month, _end_hour, _end_min, onSuccess, onFail, onError)', function () {
            it('should successfully add an discount', function (done) {
                discountService.addDiscount(token, locationId, 25, [], null, '2017', '1', '2', '11', '15', '2017', '2', '1', '23', '0',
                    function (discount, location) {
                        var start = momentTz.tz(discount.start_timestamp, location.local_timezone);
                        var end = momentTz.tz(discount.end_timestamp, location.local_timezone);

                        assert.equal(start.year(), '2017');
                        assert.equal((start.month() + 1), '1'); // +1 because the moment lib starts counting months from 0
                        assert.equal(start.date(), '2');
                        assert.equal(start.hours(), '11');
                        assert.equal(start.minutes(), '15');

                        assert.equal(discount.start_year, '2017');
                        assert.equal(discount.start_month, '1'); // +1 because the moment lib starts counting months from 0
                        assert.equal(discount.start_day_of_month, '2');
                        assert.equal(discount.start_hour, '11');
                        assert.equal(discount.start_min, '15');

                        assert.equal(end.year(), '2017');
                        assert.equal((end.month() + 1), '2'); // +1 because the moment lib ends counting months from 0
                        assert.equal(end.date(), '1');
                        assert.equal(end.hours(), '23');
                        assert.equal(end.minutes(), '0');

                        assert.equal(discount.end_year, '2017');
                        assert.equal(discount.end_month, '2'); // +1 because the moment lib ends counting months from 0
                        assert.equal(discount.end_day_of_month, '1');
                        assert.equal(discount.end_hour, '23');
                        assert.equal(discount.end_min, '0');

                        assert.equal(location.id, locationId);
                        assert.equal(discount.location_id, locationId);
                        assert.equal(discount.items.length, 0);
                        assert.equal(discount.deleted, false);
                        assert.equal(discount.discount, 25);
                        assert.equal(discount.excluded_items, null);

                        discountService.getDiscount(discount.id,
                            function (discountCheck) {
                                var startCheck = momentTz.tz(discountCheck.start_timestamp, location.local_timezone);
                                var endCheck = momentTz.tz(discountCheck.end_timestamp, location.local_timezone);

                                assert.equal(discountCheck.id, discount.id);
                                assert.equal(startCheck.year(), '2017');
                                assert.equal((startCheck.month() + 1), '1'); // +1 because the moment lib starts counting months from 0
                                assert.equal(startCheck.date(), '2');
                                assert.equal(startCheck.hours(), '11');
                                assert.equal(startCheck.minutes(), '15');

                                assert.equal(discountCheck.start_year, '2017');
                                assert.equal(discountCheck.start_month, '1'); // +1 because the moment lib starts counting months from 0
                                assert.equal(discountCheck.start_day_of_month, '2');
                                assert.equal(discountCheck.start_hour, '11');
                                assert.equal(discountCheck.start_min, '15');

                                assert.equal(endCheck.year(), '2017');
                                assert.equal((endCheck.month() + 1), '2'); // +1 because the moment lib ends counting months from 0
                                assert.equal(endCheck.date(), '1');
                                assert.equal(endCheck.hours(), '23');
                                assert.equal(endCheck.minutes(), '0');

                                assert.equal(discountCheck.end_year, '2017');
                                assert.equal(discountCheck.end_month, '2'); // +1 because the moment lib ends counting months from 0
                                assert.equal(discountCheck.end_day_of_month, '1');
                                assert.equal(discountCheck.end_hour, '23');
                                assert.equal(discountCheck.end_min, '0');

                                assert.equal(discountCheck.items.length, 0);
                                assert.equal(discountCheck.deleted, false);
                                assert.equal(discountCheck.discount, 25);
                                assert.equal(discountCheck.excluded_items, null);
                                assert.equal(discountCheck.location_id, locationId);

                                done();
                            }, function (message) {
                                done(new Error(message));
                            }, function (err) {
                                done(err);
                            });
                    }, function (message) {
                        done(new Error(message));
                    }, function (err) {
                        done(err);
                    });
            });
        });

        describe('#deleteDiscount(_token, _discount_id, onSuccess, onFail, onError)', function () {
            it('should successfully add an discount and delete an discount', function (done) {
                discountService.addDiscount(token, locationId, 25, [], null, '2017', '1', '2', '11', '15', '2017', '2', '1', '23', '0',
                    function (discount, location) {
                        var start = momentTz.tz(discount.start_timestamp, location.local_timezone);
                        var end = momentTz.tz(discount.end_timestamp, location.local_timezone);

                        assert.equal(start.year(), '2017');
                        assert.equal((start.month() + 1), '1'); // +1 because the moment lib starts counting months from 0
                        assert.equal(start.date(), '2');
                        assert.equal(start.hours(), '11');
                        assert.equal(start.minutes(), '15');

                        assert.equal(discount.start_year, '2017');
                        assert.equal(discount.start_month, '1'); // +1 because the moment lib starts counting months from 0
                        assert.equal(discount.start_day_of_month, '2');
                        assert.equal(discount.start_hour, '11');
                        assert.equal(discount.start_min, '15');

                        assert.equal(end.year(), '2017');
                        assert.equal((end.month() + 1), '2'); // +1 because the moment lib ends counting months from 0
                        assert.equal(end.date(), '1');
                        assert.equal(end.hours(), '23');
                        assert.equal(end.minutes(), '0');

                        assert.equal(discount.end_year, '2017');
                        assert.equal(discount.end_month, '2'); // +1 because the moment lib ends counting months from 0
                        assert.equal(discount.end_day_of_month, '1');
                        assert.equal(discount.end_hour, '23');
                        assert.equal(discount.end_min, '0');

                        assert.equal(location.id, locationId);
                        assert.equal(discount.location_id, locationId);
                        assert.equal(discount.items.length, 0);
                        assert.equal(discount.deleted, false);
                        assert.equal(discount.discount, 25);
                        assert.equal(discount.excluded_items, null);

                        discountService.deleteDiscount(token, discount.id,
                            function () {
                                discountService.getDiscount(discount.id,
                                    function (discountCheck) {
                                        var startCheck = momentTz.tz(discountCheck.start_timestamp, location.local_timezone);
                                        var endCheck = momentTz.tz(discountCheck.end_timestamp, location.local_timezone);

                                        assert.equal(discountCheck.id, discount.id);
                                        assert.equal(startCheck.year(), '2017');
                                        assert.equal((startCheck.month() + 1), '1'); // +1 because the moment lib starts counting months from 0
                                        assert.equal(startCheck.date(), '2');
                                        assert.equal(startCheck.hours(), '11');
                                        assert.equal(startCheck.minutes(), '15');

                                        assert.equal(discountCheck.start_year, '2017');
                                        assert.equal(discountCheck.start_month, '1'); // +1 because the moment lib starts counting months from 0
                                        assert.equal(discountCheck.start_day_of_month, '2');
                                        assert.equal(discountCheck.start_hour, '11');
                                        assert.equal(discountCheck.start_min, '15');

                                        assert.equal(endCheck.year(), '2017');
                                        assert.equal((endCheck.month() + 1), '2'); // +1 because the moment lib ends counting months from 0
                                        assert.equal(endCheck.date(), '1');
                                        assert.equal(endCheck.hours(), '23');
                                        assert.equal(endCheck.minutes(), '0');

                                        assert.equal(discountCheck.end_year, '2017');
                                        assert.equal(discountCheck.end_month, '2'); // +1 because the moment lib ends counting months from 0
                                        assert.equal(discountCheck.end_day_of_month, '1');
                                        assert.equal(discountCheck.end_hour, '23');
                                        assert.equal(discountCheck.end_min, '0');

                                        assert.equal(discountCheck.items.length, 0);
                                        assert.equal(discountCheck.deleted, true);
                                        assert.equal(discountCheck.discount, 25);
                                        assert.equal(discountCheck.excluded_items, null);
                                        assert.equal(discountCheck.location_id, locationId);

                                        done();
                                    }, function (message) {
                                        done(new Error(message));
                                    }, function (err) {
                                        done(err);
                                    });

                            }, function (message) {
                                done(new Error(message));
                            }, function (err) {
                                done(err);
                            });
                    }, function (message) {
                        done(new Error(message));
                    }, function (err) {
                        done(err);
                    });
            });
        });

        describe('#getLocationDiscounts(_location_id, _pos, _length, _active, onSuccess, onFail, onError)', function () {
            it('should successfully add an active discount and retrieve all active discount by location', function (done) {
                /**
                 * Create an discount that should be active (with now() + 1 hour)
                 */
                var startDiscount = momentTz().tz(globalLocation.local_timezone);
                startDiscount.add(1, 'hours');

                var endDiscount = momentTz().tz(globalLocation.local_timezone);
                endDiscount.add(2, 'hours');

                discountService.addDiscount(token, locationId, 25, [], null, startDiscount.year(), (startDiscount.month() + 1), startDiscount.date(), startDiscount.hours(), startDiscount.minutes(), endDiscount.year(), (endDiscount.month() + 1), endDiscount.date(), endDiscount.hours(), endDiscount.minutes(),
                    function (checkDiscount, location) {
                        /**
                         * Verify if the added discount is valid
                         */
                        discountService.getDiscount(checkDiscount.id, function (discount) {
                            var start = momentTz.tz(discount.start_timestamp, location.local_timezone);
                            var end = momentTz.tz(discount.end_timestamp, location.local_timezone);

                            assert.equal(start.year(), startDiscount.year());
                            assert.equal((start.month() + 1), (startDiscount.month() + 1)); // +1 because the moment lib starts counting months from 0
                            assert.equal(start.date(), startDiscount.date());
                            assert.equal(start.hours(), startDiscount.hours());
                            assert.equal(start.minutes(), startDiscount.minutes());

                            assert.equal(discount.start_year, startDiscount.year());
                            assert.equal(discount.start_month, (startDiscount.month() + 1)); // +1 because the moment lib starts counting months from 0
                            assert.equal(discount.start_day_of_month, startDiscount.date());
                            assert.equal(discount.start_hour, startDiscount.hours());
                            assert.equal(discount.start_min, startDiscount.minutes());

                            assert.equal(end.year(), endDiscount.year());
                            assert.equal((end.month() + 1), (endDiscount.month() + 1)); // +1 because the moment lib ends counting months from 0
                            assert.equal(end.date(), endDiscount.date());
                            assert.equal(end.hours(), endDiscount.hours());
                            assert.equal(end.minutes(), endDiscount.minutes());

                            assert.equal(discount.end_year, endDiscount.year());
                            assert.equal(discount.end_month, (endDiscount.month() + 1)); // +1 because the moment lib ends counting months from 0
                            assert.equal(discount.end_day_of_month, endDiscount.date());
                            assert.equal(discount.end_hour, endDiscount.hours());
                            assert.equal(discount.end_min, endDiscount.minutes());

                            assert.equal(location.id, locationId);
                            assert.equal(discount.location_id, locationId);
                            assert.equal(discount.items.length, 0);
                            assert.equal(discount.deleted, false);
                            assert.equal(discount.discount, 25);
                            assert.equal(discount.excluded_items, null);
                            assert.equal(endDiscount.diff(startDiscount, 'hours'), 1);

                            discountService.getLocationDiscounts(locationId, 0, 100, true,
                                function (discounts) {
                                    // traverse to verify if the added discount is returned
                                    for(var i = 0; i < discounts.length; i++) {
                                        if(discounts[i].id == checkDiscount.id) {
                                            return done();
                                        }
                                    }

                                    done(new Error('Added discount was not found in the discounts list.'));
                                }, function (message) {
                                    done(new Error(message));
                                }, function (err) {
                                    done(err);
                                });
                        }, function (message) {
                            done(new Error(message));
                        }, function (err) {
                            done(err);
                        });
                    }, function (message) {
                        done(new Error(message));
                    }, function (err) {
                        done(err);
                    });
            });
        });

        describe('#getLocationDiscounts(_location_id, _pos, _length, _active, onSuccess, onFail, onError)', function () {
            it('should successfully add an old (inactive) discount and retrieve all active discount by location, where the inactive discount is not included', function (done) {
                /**
                 * Create an discount that should be inactive (10 days old)
                 */
                var startDiscount = momentTz().tz(globalLocation.local_timezone);
                startDiscount.subtract(10, 'days');

                var endDiscount = momentTz().tz(globalLocation.local_timezone);
                endDiscount.subtract(10, 'days');
                endDiscount.add(1, 'hours');

                discountService.addDiscount(token, locationId, 25, [], null, startDiscount.year(), (startDiscount.month() + 1), startDiscount.date(), startDiscount.hours(), startDiscount.minutes(), endDiscount.year(), (endDiscount.month() + 1), endDiscount.date(), endDiscount.hours(), endDiscount.minutes(),
                    function (checkDiscount, location) {
                        /**
                         * Verify if the added discount is valid
                         */
                        discountService.getDiscount(checkDiscount.id, function (discount) {
                            var start = momentTz.tz(discount.start_timestamp, location.local_timezone);
                            var end = momentTz.tz(discount.end_timestamp, location.local_timezone);

                            assert.equal(start.year(), startDiscount.year());
                            assert.equal((start.month() + 1), (startDiscount.month() + 1)); // +1 because the moment lib starts counting months from 0
                            assert.equal(start.date(), startDiscount.date());
                            assert.equal(start.hours(), startDiscount.hours());
                            assert.equal(start.minutes(), startDiscount.minutes());

                            assert.equal(discount.start_year, startDiscount.year());
                            assert.equal(discount.start_month, (startDiscount.month() + 1)); // +1 because the moment lib starts counting months from 0
                            assert.equal(discount.start_day_of_month, startDiscount.date());
                            assert.equal(discount.start_hour, startDiscount.hours());
                            assert.equal(discount.start_min, startDiscount.minutes());

                            assert.equal(end.year(), endDiscount.year());
                            assert.equal((end.month() + 1), (endDiscount.month() + 1)); // +1 because the moment lib ends counting months from 0
                            assert.equal(end.date(), endDiscount.date());
                            assert.equal(end.hours(), endDiscount.hours());
                            assert.equal(end.minutes(), endDiscount.minutes());

                            assert.equal(discount.end_year, endDiscount.year());
                            assert.equal(discount.end_month, (endDiscount.month() + 1)); // +1 because the moment lib ends counting months from 0
                            assert.equal(discount.end_day_of_month, endDiscount.date());
                            assert.equal(discount.end_hour, endDiscount.hours());
                            assert.equal(discount.end_min, endDiscount.minutes());

                            assert.equal(location.id, locationId);
                            assert.equal(discount.location_id, locationId);
                            assert.equal(discount.items.length, 0);
                            assert.equal(discount.deleted, false);
                            assert.equal(discount.discount, 25);
                            assert.equal(discount.excluded_items, null);
                            assert.equal(endDiscount.diff(startDiscount, 'hours'), 1);

                            discountService.getLocationDiscounts(locationId, 0, 100, true,
                                function (discounts) {
                                    // traverse to verify if the added discount is returned
                                    for(var i = 0; i < discounts.length; i++) {
                                        if(discounts[i].id == checkDiscount.id) {
                                            return done(new Error('Added discount should not be included in the active discount list'));
                                        }
                                    }

                                    done();
                                }, function (message) {
                                    done(new Error(message));
                                }, function (err) {
                                    done(err);
                                });
                        }, function (message) {
                            done(new Error(message));
                        }, function (err) {
                            done(err);
                        });
                    }, function (message) {
                        done(new Error(message));
                    }, function (err) {
                        done(err);
                    });
            });
        });

        describe('#getLocationDiscounts(_location_id, _pos, _length, _active, onSuccess, onFail, onError)', function () {
            it('should successfully add an old (inactive) discount and retrieve all active and inactive discount by location', function (done) {
                /**
                 * Create an discount that should be inactive (10 days old)
                 */
                var startDiscount = momentTz().tz(globalLocation.local_timezone);
                startDiscount.subtract(10, 'days');

                var endDiscount = momentTz().tz(globalLocation.local_timezone);
                endDiscount.subtract(10, 'days');
                endDiscount.add(1, 'hours');

                discountService.addDiscount(token, locationId, 25, [], null, startDiscount.year(), (startDiscount.month() + 1), startDiscount.date(), startDiscount.hours(), startDiscount.minutes(), endDiscount.year(), (endDiscount.month() + 1), endDiscount.date(), endDiscount.hours(), endDiscount.minutes(),
                    function (checkDiscount, location) {
                        /**
                         * Verify if the added discount is valid
                         */
                        discountService.getDiscount(checkDiscount.id, function (discount) {
                            var start = momentTz.tz(discount.start_timestamp, location.local_timezone);
                            var end = momentTz.tz(discount.end_timestamp, location.local_timezone);

                            assert.equal(start.year(), startDiscount.year());
                            assert.equal((start.month() + 1), (startDiscount.month() + 1)); // +1 because the moment lib starts counting months from 0
                            assert.equal(start.date(), startDiscount.date());
                            assert.equal(start.hours(), startDiscount.hours());
                            assert.equal(start.minutes(), startDiscount.minutes());

                            assert.equal(discount.start_year, startDiscount.year());
                            assert.equal(discount.start_month, (startDiscount.month() + 1)); // +1 because the moment lib starts counting months from 0
                            assert.equal(discount.start_day_of_month, startDiscount.date());
                            assert.equal(discount.start_hour, startDiscount.hours());
                            assert.equal(discount.start_min, startDiscount.minutes());

                            assert.equal(end.year(), endDiscount.year());
                            assert.equal((end.month() + 1), (endDiscount.month() + 1)); // +1 because the moment lib ends counting months from 0
                            assert.equal(end.date(), endDiscount.date());
                            assert.equal(end.hours(), endDiscount.hours());
                            assert.equal(end.minutes(), endDiscount.minutes());

                            assert.equal(discount.end_year, endDiscount.year());
                            assert.equal(discount.end_month, (endDiscount.month() + 1)); // +1 because the moment lib ends counting months from 0
                            assert.equal(discount.end_day_of_month, endDiscount.date());
                            assert.equal(discount.end_hour, endDiscount.hours());
                            assert.equal(discount.end_min, endDiscount.minutes());

                            assert.equal(location.id, locationId);
                            assert.equal(discount.location_id, locationId);
                            assert.equal(discount.items.length, 0);
                            assert.equal(discount.deleted, false);
                            assert.equal(discount.discount, 25);
                            assert.equal(discount.excluded_items, null);
                            assert.equal(endDiscount.diff(startDiscount, 'hours'), 1);

                            discountService.getLocationDiscounts(locationId, 0, 100, false,
                                function (discounts) {
                                    // traverse to verify if the added discount is returned
                                    for(var i = 0; i < discounts.length; i++) {
                                        if(discounts[i].id == checkDiscount.id) {
                                            return done();
                                        }
                                    }

                                    done(new Error('Added discount should be included in the discounts list'));
                                }, function (message) {
                                    done(new Error(message));
                                }, function (err) {
                                    done(err);
                                });
                        }, function (message) {
                            done(new Error(message));
                        }, function (err) {
                            done(err);
                        });
                    }, function (message) {
                        done(new Error(message));
                    }, function (err) {
                        done(err);
                    });
            });
        });

        describe('#getLocationDiscountsByRangeInclusion(_location_id, _limit, _offset, _start_year, _start_month, _start_day_of_month, _start_hour, _start_min, _end_year, _end_month, _end_day_of_month, _end_hour, _end_min, onSuccess, onFail, onError)', function () {
            it('should successfully add an discount and verify that the added discount is in a given range', function (done) {
                /**
                 * Create an discount that should be inactive (10 days old)
                 */
                var startDiscount = momentTz().tz(globalLocation.local_timezone);
                startDiscount.subtract(10, 'days');

                var endDiscount = momentTz().tz(globalLocation.local_timezone);
                endDiscount.subtract(10, 'days');
                endDiscount.add(1, 'hours');

                /**
                 * Add discount for one hour, which is 10 days old
                 */
                discountService.addDiscount(token, locationId, 25, [], null, startDiscount.year(), (startDiscount.month() + 1), startDiscount.date(), startDiscount.hours(), startDiscount.minutes(), endDiscount.year(), (endDiscount.month() + 1), endDiscount.date(), endDiscount.hours(), endDiscount.minutes(),
                    function (discount, location) {
                        discountService.getLocationDiscountsByRangeInclusion(locationId, '100', '0',
                            startDiscount.year(), (startDiscount.month() + 1), startDiscount.date(), startDiscount.hours(), startDiscount.minutes(),
                            endDiscount.year(), (endDiscount.month() + 1), endDiscount.date(), endDiscount.hours(), endDiscount.minutes(),
                            function (discounts) {
                                if (discounts.length == 0) {
                                    done(new Error('Given discounts array is empty. The array should include at least the before added discount.'));
                                }

                                /**
                                 * Verify that he added discount is in the range of the discounts array
                                 */
                                for (var i = 0; i < discounts.length; i++) {
                                    if (discounts[i].id == discount.id) {
                                        return done();
                                    }
                                }

                                return done(new Error('Given discounts array is empty. The array should include at least the before added discount.'));
                            }, function (message) {
                                done(new Error(message));
                            }, function (err) {
                                done(err);
                            }
                        )
                    }, function (message) {
                        done(new Error(message));
                    }, function (err) {
                        done(err);
                    });
            });
        });

        describe('#getLocationDiscountsByDate(_location_id, _limit, _offset, _start_year, _start_month, _start_day_of_month, _start_hour, _start_min, onSuccess, onFail, onError)', function () {
            it('should successfully add an discount and verify that the added discount is in a given range', function (done) {
                /**
                 * Create an discount that should be inactive (10 days old)
                 */
                var startDiscount = momentTz().tz(globalLocation.local_timezone);
                startDiscount.subtract(10, 'days');

                var endDiscount = momentTz().tz(globalLocation.local_timezone);
                endDiscount.subtract(10, 'days');
                endDiscount.add(1, 'hours');

                /**
                 * Add discount for one hour, which is 10 days old
                 */
                discountService.addDiscount(token, locationId, 25, [], null, startDiscount.year(), (startDiscount.month() + 1), startDiscount.date(), startDiscount.hours(), startDiscount.minutes(), endDiscount.year(), (endDiscount.month() + 1), endDiscount.date(), endDiscount.hours(), endDiscount.minutes(),
                    function (discount, location) {
                        startDiscount.add(30, 'minutes');

                        discountService.getLocationDiscountsByDate(locationId, '100', '0',
                            startDiscount.year(), (startDiscount.month() + 1), startDiscount.date(), startDiscount.hours(), startDiscount.minutes(),
                            function (discounts) {
                                if (discounts.length == 0) {
                                    done(new Error('Given discounts array is empty. The array should include at least the before added discount.'));
                                }

                                /**
                                 * Verify that he added discount is in the range of the discounts array
                                 */
                                for (var i = 0; i < discounts.length; i++) {
                                    if (discounts[i].id == discount.id) {
                                        return done();
                                    }
                                }

                                return done(new Error('Given discounts array is empty. The array should include at least the before added discount.'));
                            }, function (message) {
                                done(new Error(message));
                            }, function (err) {
                                done(err);
                            }
                        )
                    }, function (message) {
                        done(new Error(message));
                    }, function (err) {
                        done(err);
                    });
            });
        });

        describe('#getLocationDiscountsByDate(_location_id, _limit, _offset, _start_year, _start_month, _start_day_of_month, _start_hour, _start_min, onSuccess, onFail, onError)', function () {
            it('should successfully add an discount and verify that the added discount is NOT in a given range', function (done) {
                /**
                 * Create an discount that should be inactive (10 days old)
                 */
                var startDiscount = momentTz().tz(globalLocation.local_timezone);
                startDiscount.subtract(10, 'days');

                var endDiscount = momentTz().tz(globalLocation.local_timezone);
                endDiscount.subtract(10, 'days');
                endDiscount.add(1, 'hours');

                /**
                 * Add discount for one hour, which is 10 days old
                 */
                discountService.addDiscount(token, locationId, 25, [], null, startDiscount.year(), (startDiscount.month() + 1), startDiscount.date(), startDiscount.hours(), startDiscount.minutes(), endDiscount.year(), (endDiscount.month() + 1), endDiscount.date(), endDiscount.hours(), endDiscount.minutes(),
                    function (discount, location) {
                        /**
                         * 30 min older, so its not in the range
                         */
                        startDiscount.subtract(1, 'minutes');

                        discountService.getLocationDiscountsByDate(locationId, '100', '0',
                            startDiscount.year(), (startDiscount.month() + 1), startDiscount.date(), startDiscount.hours(), startDiscount.minutes(),
                            function (discounts) {
                                /**
                                 * Verify that he added discount is in the range of the discounts array
                                 */
                                for (var i = 0; i < discounts.length; i++) {
                                    if (discounts[i].id == discount.id) {
                                        return done(new Error('The given discount should not be in the range of the discounts array.'));
                                    }
                                }

                                return done();
                            }, function (message) {
                                done(new Error(message));
                            }, function (err) {
                                done(err);
                            }
                        )
                    }, function (message) {
                        done(new Error(message));
                    }, function (err) {
                        done(err);
                    });
            });
        });

        describe('#getLocationDiscountsByRangeInclusion(_location_id, _limit, _offset, _start_year, _start_month, _start_day_of_month, _start_hour, _start_min, _end_year, _end_month, _end_day_of_month, _end_hour, _end_min, onSuccess, onFail, onError)', function () {
            it('should successfully add an discount and verify that the added discount is NOT in a given range', function (done) {
                /**
                 * Create an discount that should be inactive (10 days old)
                 */
                var startDiscount = momentTz().tz(globalLocation.local_timezone);
                startDiscount.subtract(10, 'days');

                var endDiscount = momentTz().tz(globalLocation.local_timezone);
                endDiscount.subtract(10, 'days');
                endDiscount.add(1, 'hours');

                /**
                 * Add discount for one hour, which is 10 days old
                 */
                discountService.addDiscount(token, locationId, 25, [], null, startDiscount.year(), (startDiscount.month() + 1), startDiscount.date(), startDiscount.hours(), startDiscount.minutes(), endDiscount.year(), (endDiscount.month() + 1), endDiscount.date(), endDiscount.hours(), endDiscount.minutes(),
                    function (discount, location) {
                        /**
                         * Create dates that are not in the range of startDiscount and endDiscount,
                         * which is between now() and (now() + 1)
                         */
                        startDiscount = momentTz().tz(globalLocation.local_timezone);
                        endDiscount = momentTz().tz(globalLocation.local_timezone);
                        endDiscount.add(1, 'hours');

                        discountService.getLocationDiscountsByRangeInclusion(locationId, '100', '0',
                            startDiscount.year(), (startDiscount.month() + 1), startDiscount.date(), startDiscount.hours(), startDiscount.minutes(),
                            endDiscount.year(), (endDiscount.month() + 1), endDiscount.date(), endDiscount.hours(), endDiscount.minutes(),
                            function (discounts) {
                                /**
                                 * Verify that the added discount is NOT in the discounts range
                                 */
                                for (var i = 0; i < discounts.length; i++) {
                                    if (discounts[i].id == discount.id) {
                                        return done(new Error('The added discount should not be in the range of the discounts array.'));
                                    }
                                }

                                return done();
                            }, function (message) {
                                done(new Error(message));
                            }, function (err) {
                                done(err);
                            }
                        )
                    }, function (message) {
                        done(new Error(message));
                    }, function (err) {
                        done(err);
                    });
            });
        });

        describe('#getLocationDiscountsByRangeInclusion(_location_id, _limit, _offset, _start_year, _start_month, _start_day_of_month, _start_hour, _start_min, _end_year, _end_month, _end_day_of_month, _end_hour, _end_min, onSuccess, onFail, onError)', function () {
            it('should successfully add an discount and verify that the added discount is NOT in a given range', function (done) {
                /**
                 * Create an discount that should be inactive (10 days old)
                 */
                var startDiscount = momentTz().tz(globalLocation.local_timezone);
                startDiscount.subtract(10, 'days');

                var endDiscount = momentTz().tz(globalLocation.local_timezone);
                endDiscount.subtract(10, 'days');
                endDiscount.add(1, 'hours');

                /**
                 * Add discount for one hour, which is 10 days old
                 */
                discountService.addDiscount(token, locationId, 25, [], null, startDiscount.year(), (startDiscount.month() + 1), startDiscount.date(), startDiscount.hours(), startDiscount.minutes(), endDiscount.year(), (endDiscount.month() + 1), endDiscount.date(), endDiscount.hours(), endDiscount.minutes(),
                    function (discount, location) {
                        /**
                         * Create dates that are not in the range of startDiscount and endDiscount,
                         * which is between now() and (now() + 1)
                         */
                        startDiscount = momentTz().tz(globalLocation.local_timezone);
                        startDiscount.subtract(20, 'days');

                        endDiscount = momentTz().tz(globalLocation.local_timezone);
                        endDiscount.subtract(20, 'days');
                        endDiscount.add(1, 'hours');

                        discountService.getLocationDiscountsByRangeInclusion(locationId, '100', '0',
                            startDiscount.year(), (startDiscount.month() + 1), startDiscount.date(), startDiscount.hours(), startDiscount.minutes(),
                            endDiscount.year(), (endDiscount.month() + 1), endDiscount.date(), endDiscount.hours(), endDiscount.minutes(),
                            function (discounts) {
                                /**
                                 * Verify that the added discount is NOT in the discounts range
                                 */
                                for (var i = 0; i < discounts.length; i++) {
                                    if (discounts[i].id == discount.id) {
                                        return done(new Error('The added discount should not be in the range of the discounts array.'));
                                    }
                                }

                                return done();
                            }, function (message) {
                                done(new Error(message));
                            }, function (err) {
                                done(err);
                            }
                        )
                    }, function (message) {
                        done(new Error(message));
                    }, function (err) {
                        done(err);
                    });
            });
        });

        describe('#getLocationDiscountsByRangeInclusion(_location_id, _limit, _offset, _start_year, _start_month, _start_day_of_month, _start_hour, _start_min, _end_year, _end_month, _end_day_of_month, _end_hour, _end_min, onSuccess, onFail, onError)', function () {
            it('should successfully add an discount and verify that the added discount is NOT in a given range', function (done) {
                /**
                 * Create an discount that should be inactive (10 days old)
                 */
                var startDiscount = momentTz().tz(globalLocation.local_timezone);
                startDiscount.subtract(10, 'days');

                var endDiscount = momentTz().tz(globalLocation.local_timezone);
                endDiscount.subtract(10, 'days');
                endDiscount.add(5, 'hours');

                /**
                 * Add discount for one hour, which is 10 days old
                 */
                discountService.addDiscount(token, locationId, 25, [], null, startDiscount.year(), (startDiscount.month() + 1), startDiscount.date(), startDiscount.hours(), startDiscount.minutes(), endDiscount.year(), (endDiscount.month() + 1), endDiscount.date(), endDiscount.hours(), endDiscount.minutes(),
                    function (discount, location) {
                        /**
                         * Add one day to the end, in order to be out of the discount range inclusion
                         */
                        startDiscount.add(3, 'hours');
                        endDiscount.add(1, 'days');

                        discountService.getLocationDiscountsByRangeInclusion(locationId, '100', '0',
                            startDiscount.year(), (startDiscount.month() + 1), startDiscount.date(), startDiscount.hours(), startDiscount.minutes(),
                            endDiscount.year(), (endDiscount.month() + 1), endDiscount.date(), endDiscount.hours(), endDiscount.minutes(),
                            function (discounts) {
                                /**
                                 * Verify that the added discount is NOT in the discounts range
                                 */
                                for (var i = 0; i < discounts.length; i++) {
                                    if (discounts[i].id == discount.id) {
                                        return done(new Error('The added discount should not be in the range of the discounts array.'));
                                    }
                                }

                                return done();
                            }, function (message) {
                                done(new Error(message));
                            }, function (err) {
                                done(err);
                            }
                        )
                    }, function (message) {
                        done(new Error(message));
                    }, function (err) {
                        done(err);
                    });
            });
        });

    });
});