var assert = require('assert');
var chai = require('chai');
var expect = chai.expect; // we are using the "expect" style of Chai

var testService = require('./test');
var authService = require('./../auth');
var clientService = require('./../client');
var menuService = require('./../menu');

var client = 'client@test.com'
var password = '123';
var token = '';

var locationId = null;

describe('Menus', function () {
    before(function(done) {
        /**
         * Migrate the test data
         */
        testService.config.setEnvironment('localhost');
        testService.migrate(function (message) {
            console.log(message);
            setup(done);
        }, function (message) {
            console.error(message);
            return done(new Error(message));
        }, function (err) {
            console.error(err);
            return done(err);
        });
    });

    after(function(done) {
        /**
         * Drop the test data
         */
        testService.truncate(function (message) {
            console.log(message);
            done();
        }, function (message) {
            console.error(message);
            done(new Error(message));
        }, function (err) {
            console.error(err);
            done(err);
        });
    });

    /**
     * Init. the testing environment
     */
    function setup(done) {
        /**
         * set the api environment to localhost for the CI setup
         */
        authService.config.setEnvironment('localhost');
        menuService.config.setEnvironment('localhost');
        clientService.config.setEnvironment('localhost');

        /**
         * auth. this test/account
         */
        authService.authenticate(client, password, function (_token, _account) {
            token = _token;

            /**
             * Get the location id
             */
            clientService.getClientsLocations(token, function (locations) {
                if(locations.length > 0) {
                    locationId = locations[0].id;
                    done();
                } else {
                    done(new Error("No location was found to run the tests against."));
                }
            }, function (message) {
                done(new Error(message));
            }, function (err) {
                done(err);
            });
        }, function (message) {
            console.error('Account auth. failed, test suit aborted.')
            done(new Error(message));
        }, function (err) {
            console.error('Account auth. failed, test suit aborted.')
            done(err);
        });
    }

    /**
     * Test the MenuService
     */
    describe('MenuService (menus only are tested)', function () {
        describe('#addMenu(_token, _location_id, _content, _title, _start_year, _start_month, _start_day, _end_year, _end_month, _end_day, _weekdays, onSuccess, onFail, onError)', function () {
            it('should successfully create a draft menu', function (done) {
                menuService.addMenu(token, locationId, {}, { "en": "title"}, 2016, 11, 26, 2017, 11, 26, null, function (menu) {
                    if(menu) {
                        assert.equal(menu.location_id, locationId);
                        assert.equal(menu.title.en, 'title');
                        assert.equal(menu.start_year, 2016);
                        assert.equal(menu.end_year, 2017);
                        assert.equal(menu.start_month, 11);
                        assert.equal(menu.end_month, 11);
                        assert.equal(menu.start_day, 26);
                        assert.equal(menu.end_day, 26);
                        assert.equal(menu.menu_status, 'draft');
                        done();
                    } else {
                        done(new Error('No menu object was found.'));
                    }
                }, function (message) {
                    console.error(message);
                    done(new Error(message));
                }, function (err) {
                    console.error(err);
                    done(err);
                });
            });
        });

        describe('#addMenu(_token, _location_id, _content, _title, _start_year, _start_month, _start_day, _end_year, _end_month, _end_day, _weekdays, onSuccess, onFail, onError)', function () {
            it('should fail to create a menu due to invalid location id', function (done) {
                menuService.addMenu(token, 'some wrong id', {}, { "en": "title"}, 2016, 11, 26, 2017, 11, 26, null, function (menu) {
                    done(new Error('The success method should never be called, due to invalid location_id.'));
                }, function (message) {
                    done();
                }, function (err) {
                    console.error(err);
                    done(err);
                });
            });
        });

        describe('#addMenu(_token, _location_id, _content, _title, _start_year, _start_month, _start_day, _end_year, _end_month, _end_day, _weekdays, onSuccess, onFail, onError)', function () {
            it('should fail to create a menu due to null location id', function (done) {
                menuService.addMenu(token, null, {}, { "en": "title"}, 2016, 11, 26, 2017, 11, 26, null, function (menu) {
                    done(new Error('The success method should never be called, due to invalid location_id.'));
                }, function (message) {
                    done();
                }, function (err) {
                    console.error(err);
                    done(err);
                });
            });
        });

        describe('#addMenu(_token, _location_id, _content, _title, _start_year, _start_month, _start_day, _end_year, _end_month, _end_day, _weekdays, onSuccess, onFail, onError)', function () {
            it('should create a menu and change its default draft status to published and back again to draft', function (done) {
                menuService.addMenu(token, locationId, {}, { "en": "title"}, 2016, 11, 26, 2017, 11, 26, null, function (menu) {
                    if(menu) {
                        // add an new menu, which is a draft by default
                        assert.equal(menu.menu_status, 'draft');

                        // change the menu to published
                        menuService.setMenuStatus(token, menu.id, 'published',
                            function (publicMenu) {
                                assert.equal(menu.id, publicMenu.id);
                                assert.equal(publicMenu.menu_status, 'published');

                                /**
                                 * and change it back again to draft
                                 */
                                menuService.setMenuStatus(token, menu.id, 'draft',
                                    function (draftMenu) {
                                        assert.equal(menu.id, draftMenu.id);
                                        assert.equal(draftMenu.menu_status, 'draft');
                                        done();
                                    }, function (message) {
                                        done(new Error(message));
                                    }, function (err) {
                                        done(err);
                                    });
                        }, function (message) {
                            done(new Error(message));
                        }, function (err) {
                            done(err);
                        });
                    } else {
                        done(new Error('No menu object was found.'));
                    }
                }, function (message) {
                    console.error(message);
                    done(new Error(message));
                }, function (err) {
                    console.error(err);
                    done(err);
                });
            });
        });

        describe('#addMenu(_token, _location_id, _content, _title, _start_year, _start_month, _start_day, _end_year, _end_month, _end_day, _weekdays, onSuccess, onFail, onError)', function () {
            it('should create a menu and fail trying to update its state due to invalid status', function (done) {
                menuService.addMenu(token, locationId, {}, { "en": "title"}, 2016, 11, 26, 2017, 11, 26, null, function (menu) {
                    if(menu) {
                        // add an new menu, which is a draft by default
                        assert.equal(menu.menu_status, 'draft');

                        // change the menu to published
                        menuService.setMenuStatus(token, menu.id, 'someWrongStatus',
                            function (publicMenu) {
                                done(new Error('Changing the status should fail due to invalid status.'));
                            }, function (message) {
                                done();
                            }, function (err) {
                                done(err);
                            });
                    } else {
                        done(new Error('No menu object was found.'));
                    }
                }, function (message) {
                    console.error(message);
                    done(new Error(message));
                }, function (err) {
                    console.error(err);
                    done(err);
                });
            });
        });
    });
});