var assert = require('assert');
var chai = require('chai');
var expect = chai.expect; // we are using the "expect" style of Chai

var testService = require('./test');
var authService = require('./../auth');
var clientService = require('./../client');
var menuService = require('./../menu');

var client = 'client@test.com'
var password = '123';
var token = '';

var locationId = null;
var menus = null;
var menu = null;
var card = null;
var category = null;

describe('Menus', function () {
    before(function(done) {
        /**
         * Migrate the test data
         */
        testService.config.setEnvironment('localhost');
        testService.migrate(function (message) {
            console.log(message);
            setup(done);
        }, function (message) {
            console.error(message);
            return done(new Error(message));
        }, function (err) {
            console.error(err);
            return done(err);
        });
    });

    after(function(done) {
        /**
         * Drop the test data
         */
        testService.truncate(function (message) {
            console.log(message);
            done();
        }, function (message) {
            console.error(message);
            done(new Error(message));
        }, function (err) {
            console.error(err);
            done(err);
        });
    });

    /**
     * Init. the testing environment
     */
    function setup(done) {
        /**
         * set the api environment to localhost for the CI setup
         */
        authService.config.setEnvironment('localhost');
        menuService.config.setEnvironment('localhost');
        clientService.config.setEnvironment('localhost');

        /**
         * auth. this test/account
         */
        authService.authenticate(client, password, function (_token, _account) {
            token = _token;

            /**
             * Get the location id
             */
            clientService.getClientsLocations(token, function (locations) {
                if(locations.length > 0) {
                    locationId = locations[0].id;

                    /**
                     * Get the menus, card, category from the location
                     */
                    menuService.getMenusByLocation(locationId, null, null, function (m) {
                        if(Array.isArray(m) && m.length > 0) {
                            menus = m;
                            menu = m[0];

                            menuService.getMenuCard(menu.id, function (c) {
                                card = c;

                                if(Array.isArray(card.categories) && card.categories.length > 0) {
                                    category = card.categories[0];
                                    done();
                                } else {
                                    done(new Error('No categories were found to run against the test suit.'));
                                }
                            }, function (message) {
                                done(new Error(message));
                            }, function (err) {
                                done(err);
                            });
                        } else {
                            done(new Error('No menus were found to run against the test suit.'));
                        }
                    }, function (message) {
                        done(new Error(message))
                    }, function (err) {
                        done(err);
                    });
                } else {
                    done(new Error("No location was found to run the tests against."));
                }
            }, function (message) {
                done(new Error(message));
            }, function (err) {
                done(err);
            });
        }, function (message) {
            console.error('Account auth. failed, test suit aborted.')
            done(new Error(message));
        }, function (err) {
            console.error('Account auth. failed, test suit aborted.')
            done(err);
        });
    }

    /**
     * Test the MenuService
     */
    describe('MenuService (menuItems only are tested)', function () {

        describe('Insert', function () {
            describe('#addMenuItem(_token, _category_id, _content, _title, _price, _currency, _description, _calories, _fats, _proteins, _carbohydrates, _tastes, _cuisine, _beverage_preferences, _food_preferences, _allergies, _tags, _images, _front_image, _is_menu, _optional_items, _selection_items, onSuccess, onFail, onError)', function () {
                it('should successfully add a menuItem', function (done) {
                    menuService.addMenuItem(token, category.id, {}, {
                            "de": "Mini Burger",
                            "en": "Mini Burger"
                        }, 1090, 'EUR', {
                            "en": "The best burger in Town",
                            "de": "Bester Burger in Wien"
                        }, 1000, 40, 35, 60, null, null, null, null, null, {"en": ["Burger"], "de": []}, null, null, false, [], null,
                        function (menuItem) {
                            // assert the most important parts
                            assert.equal(menuItem.price, '1090');
                            assert.equal(menuItem.currency, 'EUR');
                            assert.equal(menuItem.menu_category_id, category.id);
                            assert.equal(menuItem.menu_child_id, null);
                            assert.equal(menuItem.title.de, 'Mini Burger');
                            assert.equal(menuItem.title.en, 'Mini Burger');
                            done();
                        }, function (message) {
                            done(new Error(message));
                        }, function (err) {
                            done(err);
                        });
                });
            });

            describe('#addMenuItem(_token, _category_id, _content, _title, _price, _currency, _description, _calories, _fats, _proteins, _carbohydrates, _tastes, _cuisine, _beverage_preferences, _food_preferences, _allergies, _tags, _images, _front_image, _is_menu, _optional_items, _selection_items, onSuccess, onFail, onError)', function () {
                it('should successfully add a menuItem with price as string', function (done) {
                    menuService.addMenuItem(token, category.id, {}, {
                            "de": "Mini Burger",
                            "en": "Mini Burger"
                        }, '1090', 'EUR', {
                            "en": "The best burger in Town",
                            "de": "Bester Burger in Wien"
                        }, 1000, 40, 35, 60, null, null, null, null, null, {"en": ["Burger"], "de": []}, null, null, false, [], null,
                        function (menuItem) {
                            // assert the most important parts
                            assert.equal(menuItem.price, '1090');
                            assert.equal(menuItem.currency, 'EUR');
                            assert.equal(menuItem.menu_category_id, category.id);
                            assert.equal(menuItem.menu_child_id, null);
                            assert.equal(menuItem.title.de, 'Mini Burger');
                            assert.equal(menuItem.title.en, 'Mini Burger');
                            done();
                        }, function (message) {
                            done(new Error(message));
                        }, function (err) {
                            done(err);
                        });
                });
            });

            describe('#addMenuItem(_token, _category_id, _content, _title, _price, _currency, _description, _calories, _fats, _proteins, _carbohydrates, _tastes, _cuisine, _beverage_preferences, _food_preferences, _allergies, _tags, _images, _front_image, _is_menu, _optional_items, _selection_items, onSuccess, onFail, onError)', function () {
                it('adding menuItem should fail due to invalid price', function (done) {
                    menuService.addMenuItem(token, category.id, {}, {
                            "de": "Mini Burger",
                            "en": "Mini Burger"
                        }, 'WrongPrice', 'EUR', {
                            "en": "The best burger in Town",
                            "de": "Bester Burger in Wien"
                        }, 1000, 40, 35, 60, null, null, null, null, null, {"en": ["Burger"], "de": []}, null, null, false, [], null,
                        function (menuItem) {
                            done(new Error('Should fail due to invalid price args.'));
                        }, function (message) {
                            done();
                        }, function (err) {
                            done(err);
                        });
                });
            });

            describe('#addMenuItem(_token, _category_id, _content, _title, _price, _currency, _description, _calories, _fats, _proteins, _carbohydrates, _tastes, _cuisine, _beverage_preferences, _food_preferences, _allergies, _tags, _images, _front_image, _is_menu, _optional_items, _selection_items, onSuccess, onFail, onError)', function () {
                it('adding menuItem should fail due to invalid valute', function (done) {
                    menuService.addMenuItem(token, category.id, {}, {
                            "de": "Mini Burger",
                            "en": "Mini Burger"
                        }, '1090', 'InvalidValute', {
                            "en": "The best burger in Town",
                            "de": "Bester Burger in Wien"
                        }, 1000, 40, 35, 60, null, null, null, null, null, {"en": ["Burger"], "de": []}, null, null, false, [], null,
                        function (menuItem) {
                            done(new Error('Should fail due to invalid valute args.'));
                        }, function (message) {
                            done();
                        }, function (err) {
                            done(err);
                        });
                });
            });

            describe('#addMenuItem(_token, _category_id, _content, _title, _price, _currency, _description, _calories, _fats, _proteins, _carbohydrates, _tastes, _cuisine, _beverage_preferences, _food_preferences, _allergies, _tags, _images, _front_image, _is_menu, _optional_items, _selection_items, onSuccess, onFail, onError)', function () {
                it('should fail due to invalid optional_items arg.', function (done) {
                    menuService.addMenuItem(token, category.id, {}, {
                            "de": "Mini Burger",
                            "en": "Mini Burger"
                        }, '1090', 'EUR', {
                            "en": "The best burger in Town",
                            "de": "Bester Burger in Wien"
                        }, 1000, 40, 35, 60, null, null, null, null, null, {"en": ["Burger"], "de": []}, null, null, false, null, null,
                        function (menuItem) {
                            done(new Error('Should fail due to invalid optional_items arg.'));
                        }, function (message) {
                            done();
                        }, function (err) {
                            done(err);
                        });
                });
            });
        });

        describe('Update', function () {
            describe('#addMenuItem(_token, _category_id, _content, _title, _price, _currency, _description, _calories, _fats, _proteins, _carbohydrates, _tastes, _cuisine, _beverage_preferences, _food_preferences, _allergies, _tags, _images, _front_image, _is_menu, _optional_items, _selection_items, onSuccess, onFail, onError)', function () {
                it('should successfully add and update an menuItem', function (done) {
                    menuService.addMenuItem(token, category.id, {}, {
                            "de": "Mini Burger",
                            "en": "Mini Burger"
                        }, 1090, 'EUR', {
                            "en": "The best burger in Town",
                            "de": "Bester Burger in Wien"
                        }, 1000, 40, 35, 60, null, null, null, null, null, {
                            "en": ["Burger"],
                            "de": []
                        }, null, null, false, [], null,
                        function (addedMenuItem) {
                            // assert the most important parts
                            assert.equal(addedMenuItem.price, '1090');
                            assert.equal(addedMenuItem.currency, 'EUR');
                            assert.equal(addedMenuItem.menu_category_id, category.id);
                            assert.equal(addedMenuItem.menu_child_id, null);
                            assert.equal(addedMenuItem.title.de, 'Mini Burger');
                            assert.equal(addedMenuItem.title.en, 'Mini Burger');

                            menuService.updateMenuItem(token, addedMenuItem.id, category.id, {}, {
                                    "de": "Big Burger",
                                    "en": "Big Burger"
                                }, '1500', 'EUR', {
                                    "en": "The best burger in Town",
                                    "de": "Bester Burger in Wien"
                                }, 1000, 40, 35, 60, null, null, null, null, null, {
                                    "en": ["Burger"],
                                    "de": []
                                }, null, null, false, [], null,
                                function (oldAddedMenuItem, updatedMenuItem) {
                                    assert.equal(addedMenuItem.price, oldAddedMenuItem.price);
                                    assert.equal(addedMenuItem.currency, oldAddedMenuItem.currency);
                                    assert.equal(addedMenuItem.menu_category_id, oldAddedMenuItem.menu_category_id);
                                    assert.equal(addedMenuItem.menu_child_id, null);
                                    assert.equal(addedMenuItem.title.de, oldAddedMenuItem.title.de);
                                    assert.equal(addedMenuItem.title.en, oldAddedMenuItem.title.de);

                                    // verify the update, oldAddedMenuItem.menu_child_id == updatedMenuItem.id
                                    assert.equal(oldAddedMenuItem.menu_child_id, updatedMenuItem.id);
                                    assert.equal(updatedMenuItem.menu_child_id, null);

                                    // verify the changes
                                    assert.equal(updatedMenuItem.price, '1500');
                                    assert.equal(updatedMenuItem.currency, oldAddedMenuItem.currency);
                                    assert.equal(updatedMenuItem.menu_category_id, oldAddedMenuItem.menu_category_id);
                                    assert.equal(updatedMenuItem.menu_child_id, null);
                                    assert.equal(updatedMenuItem.title.de, 'Big Burger');
                                    assert.equal(updatedMenuItem.title.en, 'Big Burger');
                                    
                                    done();
                                }, function (message) {
                                    done(new Error(message));
                                }, function (err) {
                                    done(err);
                                });
                        }, function (message) {
                            done(new Error(message));
                        }, function (err) {
                            done(err);
                        });
                });
            });
        });

    });
});