var assert = require('assert');
var chai = require('chai');
var expect = chai.expect; // we are using the "expect" style of Chai

var testService = require('./test');
var userService = require('./../user');
var basketService = require('./../basket');
var authService = require('./../auth');
var clientService = require('./../client');
var menuService = require('./../menu');

var client = 'client@test.com'
var clientId = '';
var password = '123';
var token = '';

var locationId = null;
var menus = null;
var menu = null;
var card = null;
var category = null;

describe('Users & Orders', function() {
    before(function(done) {
        /**
         * Migrate the test data
         */
        testService.config.setEnvironment('localhost');
        testService.migrate(function (message) {
            console.log(message);
            setup(done);
        }, function (message) {
            console.error(message);
            return done(new Error(message));
        }, function (err) {
            console.error(err);
            return done(err);
        });
    });

    after(function(done) {
        /**
         * Drop the test data
         */
        testService.truncate(function (message) {
            console.log(message);
            done();
        }, function (message) {
            console.error(message);
            done(new Error(message));
        }, function (err) {
            console.error(err);
            done(err);
        });
    });

    /**
     * Init. the testing environment
     */
    function setup(done) {
        /**
         * set the api enviroment to localhost for the CI setup
         */
        userService.config.setEnvironment('localhost');
        basketService.config.setEnvironment('localhost');
        authService.config.setEnvironment('localhost');
        menuService.config.setEnvironment('localhost');
        clientService.config.setEnvironment('localhost');


        /**
         * auth. this test/account
         */
        authService.authenticate(client, password, function (_token, _account) {
            token = _token;
            clientId = _account.id;

            /**
             * Get the location id
             */
            clientService.getClientsLocations(token, function (locations) {
                if (locations.length > 0) {
                    locationId = locations[0].id;

                    /**
                     * Get the menus, card, category from the location
                     */
                    menuService.getMenusByLocation(locationId, null, null, function (m) {
                        if (Array.isArray(m) && m.length > 0) {
                            menus = m;
                            menu = m[0];

                            menuService.getMenuCard(menu.id, function (c) {
                                card = c;

                                if (Array.isArray(card.categories) && card.categories.length > 0) {
                                    category = card.categories[0];
                                    done();
                                } else {
                                    done(new Error('No categories were found to run against the test suit.'));
                                }
                            }, function (message) {
                                done(new Error(message));
                            }, function (err) {
                                done(err);
                            });
                        } else {
                            done(new Error('No menus were found to run against the test suit.'));
                        }
                    }, function (message) {
                        done(new Error(message))
                    }, function (err) {
                        done(err);
                    });
                } else {
                    done(new Error("No location was found to run the tests against."));
                }
            }, function (message) {
                done(new Error(message));
            }, function (err) {
                done(err);
            });
        }, function (message) {
            console.error('Account auth. failed, test suit aborted.')
            done(new Error(message));
        }, function (err) {
            console.error('Account auth. failed, test suit aborted.')
            done(err);
        });
    }

    /**
     * Test the userService and ordering
     */
    describe('UserService', function() {
        describe('#placePublicOrder(_location_id, _basket, onSuccess, onFail, onError)', function() {
            it('should successfully place a public order for reservation and no pre-order', function (done) {
                var basket = basketService.instance();

                basket.setNotes('I may post some stuff on facebook');
                basket.setOrderType(basketService.reservationType());
                basket.setFirstName('Max');
                basket.setLastName('Muster');
                basket.setMail('max@muster.com');
                basket.setTelNumber('1111111');
                basket.setNumberOfPeople('3');
                basket.setDay('22');
                basket.setMonth('1');
                basket.setYear('2017');
                basket.setHours('17');
                basket.setMinutes('0');

                userService.placePublicOrder(locationId, basket, function (order, items) {
                    assert.equal(items.length, 0);
                    assert.equal(order.notes, basket.notes);
                    assert.equal(order.order_type, basketService.reservationType());
                    assert.equal(order.guest.first_name, basket.firstName);
                    assert.equal(order.guest.last_name, basket.lastName);
                    assert.equal(order.guest.mail, basket.mail);
                    assert.equal(order.guest.tel_number, basket.telNumber);
                    assert.equal(order.day_of_month, basket.day);
                    assert.equal(order.month, basket.month);
                    assert.equal(order.year, basket.year);
                    assert.equal(order.client_id, clientId);
                    assert.equal(order.location_id, locationId);
                    assert.equal(order.order_status_id, 'pending');
                    // user id must be null due to public order
                    assert.equal(order.user_id, null);
                    assert.equal(order.local_start_hour, basket.hour);
                    assert.equal(order.local_start_min, basket.min);

                    done();
                }, function (message) {
                    done(new Error(message));
                }, function (err) {
                    done(err);
                });
            });
        });

        describe('#placePublicOrder(_location_id, _basket, onSuccess, onFail, onError)', function() {
            it('should successfully place a public order for reservation and no pre-order and should be confirmed by an authenticated client', function (done) {
                var basket = basketService.instance();

                basket.setNotes('I may post some stuff on facebook');
                basket.setOrderType(basketService.reservationType());
                basket.setFirstName('Max');
                basket.setLastName('Muster');
                basket.setMail('max@muster.com');
                basket.setTelNumber('1111111');
                basket.setNumberOfPeople('3');
                basket.setDay('22');
                basket.setMonth('1');
                basket.setYear('2017');
                basket.setHours('17');
                basket.setMinutes('0');

                userService.placePublicOrder(locationId, basket, function (order, items) {
                    assert.equal(items.length, 0);
                    assert.equal(order.notes, basket.notes);
                    assert.equal(order.order_type, basketService.reservationType());
                    assert.equal(order.guest.first_name, basket.firstName);
                    assert.equal(order.guest.last_name, basket.lastName);
                    assert.equal(order.guest.mail, basket.mail);
                    assert.equal(order.guest.tel_number, basket.telNumber);
                    assert.equal(order.day_of_month, basket.day);
                    assert.equal(order.month, basket.month);
                    assert.equal(order.year, basket.year);
                    assert.equal(order.client_id, clientId);
                    assert.equal(order.location_id, locationId);
                    assert.equal(order.order_status_id, 'pending');
                    // user id must be null due to public order
                    assert.equal(order.user_id, null);
                    assert.equal(order.local_start_hour, basket.hour);
                    assert.equal(order.local_start_min, basket.min);

                    clientService.confirmOrder(token, order.id, function (_confirmedOrder) {
                        assert.equal(order.id, _confirmedOrder.id);
                        assert.equal(_confirmedOrder.order_status_id, 'confirmed');
                        done();
                    }, function (message) {
                        done(new Error(message));
                    }, function (err) {
                        done(err);
                    });
                }, function (message) {
                    done(new Error(message));
                }, function (err) {
                    done(err);
                });
            });
        });

        describe('#placePublicOrder(_location_id, _basket, onSuccess, onFail, onError)', function() {
            it('should successfully place a public order for reservation and no pre-order and should be denied by an authenticated client', function (done) {
                var basket = basketService.instance();

                basket.setNotes('I may post some stuff on facebook');
                basket.setOrderType(basketService.reservationType());
                basket.setFirstName('Max');
                basket.setLastName('Muster');
                basket.setMail('max@muster.com');
                basket.setTelNumber('1111111');
                basket.setNumberOfPeople('3');
                basket.setDay('22');
                basket.setMonth('1');
                basket.setYear('2017');
                basket.setHours('17');
                basket.setMinutes('0');

                userService.placePublicOrder(locationId, basket, function (order, items) {
                    assert.equal(items.length, 0);
                    assert.equal(order.notes, basket.notes);
                    assert.equal(order.order_type, basketService.reservationType());
                    assert.equal(order.guest.first_name, basket.firstName);
                    assert.equal(order.guest.last_name, basket.lastName);
                    assert.equal(order.guest.mail, basket.mail);
                    assert.equal(order.guest.tel_number, basket.telNumber);
                    assert.equal(order.day_of_month, basket.day);
                    assert.equal(order.month, basket.month);
                    assert.equal(order.year, basket.year);
                    assert.equal(order.client_id, clientId);
                    assert.equal(order.location_id, locationId);
                    assert.equal(order.order_status_id, 'pending');
                    // user id must be null due to public order
                    assert.equal(order.user_id, null);
                    assert.equal(order.local_start_hour, basket.hour);
                    assert.equal(order.local_start_min, basket.min);

                    clientService.denyOrder(token, order.id, function (_confirmedOrder) {
                        assert.equal(order.id, _confirmedOrder.id);
                        assert.equal(_confirmedOrder.order_status_id, 'denied');
                        done();
                    }, function (message) {
                        done(new Error(message));
                    }, function (err) {
                        done(err);
                    });
                }, function (message) {
                    done(new Error(message));
                }, function (err) {
                    done(err);
                });
            });
        });

        describe('#placePublicOrder(_location_id, _basket, onSuccess, onFail, onError)', function() {
            it('should successfully place a public order for reservation and no pre-order and should be confirmed by an public client', function (done) {
                var basket = basketService.instance();

                basket.setNotes('I may post some stuff on facebook');
                basket.setOrderType(basketService.reservationType());
                basket.setFirstName('Max');
                basket.setLastName('Muster');
                basket.setMail('max@muster.com');
                basket.setTelNumber('1111111');
                basket.setNumberOfPeople('3');
                basket.setDay('22');
                basket.setMonth('1');
                basket.setYear('2017');
                basket.setHours('17');
                basket.setMinutes('0');

                userService.placePublicOrder(locationId, basket, function (order, items) {
                    assert.equal(items.length, 0);
                    assert.equal(order.notes, basket.notes);
                    assert.equal(order.order_type, basketService.reservationType());
                    assert.equal(order.guest.first_name, basket.firstName);
                    assert.equal(order.guest.last_name, basket.lastName);
                    assert.equal(order.guest.mail, basket.mail);
                    assert.equal(order.guest.tel_number, basket.telNumber);
                    assert.equal(order.day_of_month, basket.day);
                    assert.equal(order.month, basket.month);
                    assert.equal(order.year, basket.year);
                    assert.equal(order.client_id, clientId);
                    assert.equal(order.location_id, locationId);
                    assert.equal(order.order_status_id, 'pending');
                    // user id must be null due to public order
                    assert.equal(order.user_id, null);
                    assert.equal(order.local_start_hour, basket.hour);
                    assert.equal(order.local_start_min, basket.min);

                    clientService.publicOrderConfirm(order.id, function () {
                        clientService.getOrder(token, order.id, function (_confirmedOrder, coupon, items) {
                            assert.equal(order.id, _confirmedOrder.id);
                            assert.equal(_confirmedOrder.order_status_id, 'confirmed');

                            done();
                        }, function (message) {
                            done(new Error(message));
                        }, function (err) {
                            done(err);
                        });
                    }, function (message) {
                        done(new Error(message));
                    }, function (err) {
                        done(err);
                    });
                }, function (message) {
                    done(new Error(message));
                }, function (err) {
                    done(err);
                });
            });
        });

        describe('#placePublicOrder(_location_id, _basket, onSuccess, onFail, onError)', function() {
            it('should successfully place a public order for reservation and no pre-order and should be denied by an public client', function (done) {
                var basket = basketService.instance();

                basket.setNotes('I may post some stuff on facebook');
                basket.setOrderType(basketService.reservationType());
                basket.setFirstName('Max');
                basket.setLastName('Muster');
                basket.setMail('max@muster.com');
                basket.setTelNumber('1111111');
                basket.setNumberOfPeople('3');
                basket.setDay('22');
                basket.setMonth('1');
                basket.setYear('2017');
                basket.setHours('17');
                basket.setMinutes('0');

                userService.placePublicOrder(locationId, basket, function (order, items) {
                    assert.equal(items.length, 0);
                    assert.equal(order.notes, basket.notes);
                    assert.equal(order.order_type, basketService.reservationType());
                    assert.equal(order.guest.first_name, basket.firstName);
                    assert.equal(order.guest.last_name, basket.lastName);
                    assert.equal(order.guest.mail, basket.mail);
                    assert.equal(order.guest.tel_number, basket.telNumber);
                    assert.equal(order.day_of_month, basket.day);
                    assert.equal(order.month, basket.month);
                    assert.equal(order.year, basket.year);
                    assert.equal(order.client_id, clientId);
                    assert.equal(order.location_id, locationId);
                    assert.equal(order.order_status_id, 'pending');
                    // user id must be null due to public order
                    assert.equal(order.user_id, null);
                    assert.equal(order.local_start_hour, basket.hour);
                    assert.equal(order.local_start_min, basket.min);

                    clientService.publicOrderDeny(order.id, function () {
                        clientService.getOrder(token, order.id, function (_confirmedOrder, coupon, items) {
                            assert.equal(order.id, _confirmedOrder.id);
                            assert.equal(_confirmedOrder.order_status_id, 'denied');

                            done();
                        }, function (message) {
                            done(new Error(message));
                        }, function (err) {
                            done(err);
                        });
                    }, function (message) {
                        done(new Error(message));
                    }, function (err) {
                        done(err);
                    });
                }, function (message) {
                    done(new Error(message));
                }, function (err) {
                    done(err);
                });
            });
        });

        describe('#placePublicOrder(_location_id, _basket, onSuccess, onFail, onError)', function() {
            it('reservation order should fail due to invalid reservation time', function (done) {
                var basket = basketService.instance();

                basket.setNotes('I may post some stuff on facebook');
                basket.setOrderType(basketService.reservationType());
                basket.setFirstName('Max');
                basket.setLastName('Muster');
                basket.setMail('max@muster.com');
                basket.setTelNumber('1111111');
                basket.setNumberOfPeople('3');
                basket.setDay('22');
                basket.setMonth('1');
                basket.setYear('2017');
                basket.setHours('0');
                basket.setMinutes('0');

                userService.placePublicOrder(locationId, basket, function (order, items) {
                    done(new Error('Reservation order should fail due to invalid reservation time.'));
                }, function (message) {
                    done();
                }, function (err) {
                    done(err);
                });
            });
        });

        describe('#placePublicOrder(_location_id, _basket, onSuccess, onFail, onError)', function() {
            it('should fail due to missing items for takeAway order', function (done) {
                var basket = basketService.instance();

                basket.setNotes('I may post some stuff on facebook');
                basket.setOrderType(basketService.takeAwayType());
                basket.setFirstName('Max');
                basket.setLastName('Muster');
                basket.setMail('max@muster.com');
                basket.setTelNumber('1111111');
                basket.setNumberOfPeople('3');
                basket.setDay('22');
                basket.setMonth('1');
                basket.setYear('2017');
                basket.setHours('17');
                basket.setMinutes('0');

                userService.placePublicOrder(locationId, basket, function (order, items) {
                    done(new Error('should fail due to missing items for takeAway order'));
                }, function (message) {
                    done();
                }, function (err) {
                    done(err);
                });
            });
        });

        describe('#placePublicOrder(_location_id, _basket, onSuccess, onFail, onError)', function() {
            it('placing public order should fail due to invalid order type', function (done) {
                var basket = basketService.instance();

                basket.setNotes('I may post some stuff on facebook');
                basket.setOrderType('Blub');
                basket.setFirstName('Max');
                basket.setLastName('Muster');
                basket.setMail('max@muster.com');
                basket.setTelNumber('1111111');
                basket.setNumberOfPeople('3');
                basket.setDay('22');
                basket.setMonth('1');
                basket.setYear('2017');
                basket.setHours('17');
                basket.setMinutes('0');

                userService.placePublicOrder(locationId, basket, function (order, items) {
                    done(new Error('Placing public order should fail due to invalid order type.'));
                }, function (message) {
                    done();
                }, function (err) {
                    done(err);
                });
            });
        });

        describe('#placePublicOrder(_location_id, _basket, onSuccess, onFail, onError)', function() {
            it('placing public order should fail due to missing mail', function (done) {
                var basket = basketService.instance();

                basket.setNotes('I may post some stuff on facebook');
                basket.setOrderType(basketService.reservationType());
                basket.setFirstName('Max');
                basket.setLastName('Muster');
                basket.setTelNumber('1111111');
                basket.setNumberOfPeople('3');
                basket.setDay('22');
                basket.setMonth('1');
                basket.setYear('2017');
                basket.setHours('17');
                basket.setMinutes('0');

                userService.placePublicOrder(locationId, basket, function (order, items) {
                    done(new Error('placing public order should fail due to missing mail.'));
                }, function (message) {
                    done();
                }, function (err) {
                    done(err);
                });
            });
        });

        describe('#placePublicOrder(_location_id, _basket, onSuccess, onFail, onError)', function() {
            it('placing public order should fail due to missing tel number', function (done) {
                var basket = basketService.instance();

                basket.setNotes('I may post some stuff on facebook');
                basket.setOrderType(basketService.reservationType());
                basket.setFirstName('Max');
                basket.setLastName('Muster');
                basket.setMail('max@muster.com');
                basket.setNumberOfPeople('3');
                basket.setDay('22');
                basket.setMonth('1');
                basket.setYear('2017');
                basket.setHours('17');
                basket.setMinutes('0');

                userService.placePublicOrder(locationId, basket, function (order, items) {
                    done(new Error('placing public order should fail due to missing tel number.'));
                }, function (message) {
                    done();
                }, function (err) {
                    done(err);
                });
            });
        });

        describe('#placePublicOrder(_location_id, _basket, onSuccess, onFail, onError)', function() {
            it('placing public order should fail due to missing day', function (done) {
                var basket = basketService.instance();

                basket.setNotes('I may post some stuff on facebook');
                basket.setOrderType(basketService.reservationType());
                basket.setFirstName('Max');
                basket.setLastName('Muster');
                basket.setMail('max@muster.com');
                basket.setTelNumber('1111111');
                basket.setNumberOfPeople('3');
                basket.setMonth('1');
                basket.setYear('2017');
                basket.setHours('17');
                basket.setMinutes('0');

                userService.placePublicOrder(locationId, basket, function (order, items) {
                    done(new Error('Placing public order should fail due to missing day.'));
                }, function (message) {
                    done();
                }, function (err) {
                    done(err);
                });
            });
        });

        describe('#placePublicOrder(_location_id, _basket, onSuccess, onFail, onError)', function() {
            it('placing public order should fail due to missing month', function (done) {
                var basket = basketService.instance();

                basket.setNotes('I may post some stuff on facebook');
                basket.setOrderType(basketService.reservationType());
                basket.setFirstName('Max');
                basket.setLastName('Muster');
                basket.setMail('max@muster.com');
                basket.setTelNumber('1111111');
                basket.setNumberOfPeople('3');
                basket.setDay('1');
                basket.setYear('2017');
                basket.setHours('17');
                basket.setMinutes('0');

                userService.placePublicOrder(locationId, basket, function (order, items) {
                    done(new Error('Placing public order should fail due to missing month.'));
                }, function (message) {
                    done();
                }, function (err) {
                    done(err);
                });
            });
        });

        describe('#placePublicOrder(_location_id, _basket, onSuccess, onFail, onError)', function() {
            it('placing public order should fail due to missing year', function (done) {
                var basket = basketService.instance();

                basket.setNotes('I may post some stuff on facebook');
                basket.setOrderType(basketService.reservationType());
                basket.setFirstName('Max');
                basket.setLastName('Muster');
                basket.setMail('max@muster.com');
                basket.setTelNumber('1111111');
                basket.setNumberOfPeople('3');
                basket.setMonth('1');
                basket.setDay('1');
                basket.setHours('17');
                basket.setMinutes('0');

                userService.placePublicOrder(locationId, basket, function (order, items) {
                    done(new Error('Placing public order should fail due to missing year.'));
                }, function (message) {
                    done();
                }, function (err) {
                    done(err);
                });
            });
        });

        describe('#placePublicOrder(_location_id, _basket, onSuccess, onFail, onError)', function() {
            it('placing public order should fail due to missing time', function (done) {
                var basket = basketService.instance();

                basket.setNotes('I may post some stuff on facebook');
                basket.setOrderType(basketService.reservationType());
                basket.setFirstName('Max');
                basket.setLastName('Muster');
                basket.setMail('max@muster.com');
                basket.setTelNumber('1111111');
                basket.setNumberOfPeople('3');
                basket.setMonth('1');
                basket.setDay('1');
                basket.setYear('2017');


                userService.placePublicOrder(locationId, basket, function (order, items) {
                    done(new Error('Placing public order should fail due to missing time.'));
                }, function (message) {
                    done();
                }, function (err) {
                    done(err);
                });
            });
        });

        describe('#placePublicOrder(_location_id, _basket, onSuccess, onFail, onError)', function() {
            it('placing public order for reservation should fail due to missing number of people', function (done) {
                var basket = basketService.instance();

                basket.setNotes('I may post some stuff on facebook');
                basket.setOrderType(basketService.reservationType());
                basket.setFirstName('Max');
                basket.setLastName('Muster');
                basket.setMail('max@muster.com');
                basket.setTelNumber('1111111');
                basket.setDay('1');
                basket.setMonth('1');
                basket.setYear('2017');
                basket.setHours('17');
                basket.setMinutes('0');

                userService.placePublicOrder(locationId, basket, function (order, items) {
                    done(new Error('placing public order for reservation should fail due to missing number of people.'));
                }, function (message) {
                    done();
                }, function (err) {
                    done(err);
                });
            });
        });



    });
});