var config = require('./config');
var request = require('./request');

var userService = {};

/**
 * Export the config for the env
 */
userService.config = config;

/**
 * Use this method to get a user by token.
 * @param _token must not be null, must be a valid user token
 * @param onSuccess(user)
 * A user object has the following structure:
 * TODO: The user's structure will change
 * { id: '46b23839-836a-44b0-b38c-1ffa44f5f920',
  billing: { debit: { iban: 'Ibannumber', bic: 'BIC' } },
  contract: null,
  specification: {},
  first_name: 'Firstname',
  last_name: 'Lastname',
  meta: null,
  createdAt: null,
  updatedAt: null }

 * @param onFail (message)
 * @param onError (error)
 */
userService.getUser = function (_token, onSuccess, onFail, onError) {
    if (_token == null) {
        return onError(new Error('Invalid token was given.'));
    }

    request.getJSON(_token, config.api('/users/'),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.user);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to get orders from a user by token.
 * @param _token must not be null, must be a valid user token
 * @param _status may be null, may be pending, denied or confirmed, the status will define the type of order to return
 * @param _pos may be null, starting position of the user's orders
 * @param _length may be null, the length to return, maximum 30
 * @param onSuccess(orders) is called when successfully retrieving the user's orders
 * @param onFail (message) is called on failure
 * @param onError (error) is called on error
 */
userService.getOrders = function (_token, _status, _pos, _length, onSuccess, onFail, onError) {
    if (_token == null) {
        return onError(new Error('Invalid token was given.'));
    }

    var query = '';

    if(_pos != null && _length != null && _status != null) {
        query = '?pos=' + _pos + '&length=' + _length + '&status=' + _status;
    }

    request.getJSON(_token, config.api('/users/orders/' + query),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.orders);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to get one order by id (UUID).
 * @param _token must not be null, must be a valid user token
 * @param onSuccess(order, coupon, items) is called when successfully retrieving the user's orders
 * @param onFail (message) is called on failure
 * @param onError (error) is called on error
 */
userService.getOrder = function (_token, _order_id, onSuccess, onFail, onError) {
    if (_token == null) {
        return onError(new Error('Invalid token was given.'));
    }

    request.getJSON(_token, config.api('/users/orders/' + _order_id),
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.order, body.coupon, body.items);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to place an new pending order.
 *
 * If the given token is null, a public order will be placed, meaning the request is not placed by a registered user.
 * If the given token is NOT null, a private order will be placed, meaning the request is placed for an active registered user.
 *
 * If you place a public order (token == null) you must provide the following data to the basket:
 *  - first_name
 *  - last_name
 *  - tel_number
 *  - mail
 *
 * @param _token may be null indicating a public order.
 * @param _location_id must not be null, must be a valid location id (UUID)
 * @param _basket must not be null, must be a valid basket filled with users's data
 * @param onSuccess(order, items) is called when placing the pending order was successful. The status of the order will be pending.
 * @param onFail(message) is called when placing the order fails
 * @param onError(error) is called when placing the order errors
 */
userService.placeOrder = function (_token, _location_id, _basket, onSuccess, onFail, onError) {
    if(_location_id == null) {
        return onError(new Error('Invalid location_id was given.'));
    }

    if(_basket == null) {
        return onError(new Error('Invalid basket was given.'));
    }

    var url = (_token == null) ? config.api('/public/users/locations/'  + _location_id + '/orders') : config.api('/users/locations/'  + _location_id + '/orders');

    request.putJSON(_token,
        url,
        {
            'location_id': _location_id,
            'items': _basket.items,
            'order_type': _basket.orderType,
            'first_name': _basket.firstName,
            'last_name': _basket.lastName,
            'tel_number': _basket.telNumber,
            'mail': _basket.mail,
            'number_of_people': _basket.numberOfPeople,
            'day': _basket.day,
            'month': _basket.month,
            'year': _basket.year,
            'hour': _basket.hour,
            'min': _basket.min,
            'notes': _basket.notes
        },
        function (body) {
            if(onSuccess != null) {
                return onSuccess(body.order, body.items);
            }
        },
        function (message) {
            return onFail(message);
        },
        function (err) {
            return onError(err);
        });
};

/**
 * Use this method to place a public order.
 * @param _location_id must not be null, must be a valid location id (UUID)
 * @param _basket must not be null, must be a valid basket filled with users's data
 * @param onSuccess(order, items) is called when placing the pending order was successful. The status of the order will be pending.
 * @param onFail(message) is called when placing the order fails
 * @param onError(error) is called when placing the order errors
 */
userService.placePublicOrder = function (_location_id, _basket, onSuccess, onFail, onError) {
    userService.placeOrder(null, _location_id, _basket, onSuccess, onFail, onError);
};

module.exports = userService;
